# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for warning when closing multiple documents."""

import aeidon
import gaupol
import gtk

__all__ = ("MultiCloseDialog",)


class MultiCloseDialog(gaupol.BuilderDialog):

    """Dialog for warning when closing multiple documents."""

    _widgets = ("main_tree_view", "main_vbox", "tran_tree_view", "tran_vbox")

    def __init__(self, parent, application, pages):
        """Initialize a :class:`MultiCloseDialog` object."""
        gaupol.BuilderDialog.__init__(self, "multiclose-dialog.ui")
        self.application = application
        self.pages = tuple(pages)
        self._init_main_tree_view()
        self._init_tran_tree_view()
        self._init_sizes()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_YES)

    def _init_main_tree_view(self):
        """Initialize the main tree view."""
        store = self._init_tree_view(self._main_tree_view)
        for page in filter(lambda x: x.project.main_changed, self.pages):
            store.append((page, True, page.get_main_basename()))
        self._main_vbox.props.visible = (len(store) > 0)

    def _init_sizes(self):
        """Initialize widget sizes."""
        if self._main_vbox.props.visible:
            gaupol.util.scale_to_content(self._main_tree_view,
                                         min_nlines=2,
                                         max_nchar=40,
                                         max_nlines=6)

        if self._tran_vbox.props.visible:
            gaupol.util.scale_to_content(self._tran_tree_view,
                                         min_nlines=2,
                                         max_nchar=40,
                                         max_nlines=6)

    def _init_tran_tree_view(self):
        """Initialize the translation tree view."""
        store = self._init_tree_view(self._tran_tree_view)
        for page in filter(lambda x: x.project.tran_changed, self.pages):
            store.append((page, True, page.get_translation_basename()))
        self._tran_vbox.props.visible = (len(store) > 0)

    def _init_tree_view(self, tree_view):
        """Initialize `tree_view` and return its model."""
        store = gtk.ListStore(object, bool, str)
        tree_view.set_model(store)
        selection = tree_view.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        renderer = gtk.CellRendererToggle()
        renderer.props.activatable = True
        renderer.connect("toggled", self._on_tree_view_cell_toggled, store)
        column = gtk.TreeViewColumn("", renderer, active=1)
        tree_view.append_column(column)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("", renderer, text=2)
        tree_view.append_column(column)
        return store

    def _on_response(self, dialog, response):
        """Save the selected documents and close pages."""
        if response == gtk.RESPONSE_YES:
            map(self._save_and_close_page, self.pages)
        if response == gtk.RESPONSE_NO:
            map(lambda x: self.application.close(x, False), self.pages)

    def _on_tree_view_cell_toggled(self, renderer, row, store):
        """Toggle save document check button value."""
        store[row][1] = not store[row][1]
        store = self._main_tree_view.get_model()
        mains = [x for x in store if x[1]]
        store = self._tran_tree_view.get_model()
        trans = [x for x in store if x[1]]
        sensitive = bool(mains or trans)
        self.set_response_sensitive(gtk.RESPONSE_YES, sensitive)

    @aeidon.deco.silent(gaupol.Default)
    def _save_and_close_page(self, page):
        """Save the selected documents and close `page`."""
        store = self._main_tree_view.get_model()
        pages = [x for x in store if x[0] is page]
        if pages and pages[0][1]:
            self.application.save_main(page)
        store = self._tran_tree_view.get_model()
        pages = [x for x in store if x[0] is page]
        if pages and pages[0][1]:
            self.application.save_translation(page)
        self.application.close(page, False)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestOutputWindow(gaupol.TestCase):

    def run__window(self):
        self.window.connect("notify::visible", gtk.main_quit)
        gtk.main()

    def setup_method(self, method):
        self.conf = gaupol.conf.output_window
        self.window = gaupol.OutputWindow()
        self.window.set_output(self.get_sample_text(aeidon.formats.SUBRIP))
        self.window.show()

    def test__init_sizes(self):
        self.conf.maximized = True
        self.window = gaupol.OutputWindow()

    def test__on_close_button_clicked(self):
        self.window._close_button.clicked()

    def test__on_close_key_pressed(self):
        self.window._on_close_key_pressed()

    def test__on_delete_event(self):
        self.window.emit("delete-event", None)

    def test__on_notify_visibile(self):
        self.window.hide()
        assert not self.conf.show
        self.window.show()
        assert self.conf.show

    def test__on_window_state_event(self):
        self.window.maximize()
        self.window.unmaximize()

    def test_set_output(self):
        self.window.set_output("test")



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Base class for observable objects."""

import aeidon

__all__ = ("Observable",)


class Observable(object):

    """Base class for observable objects.

    :cvar signals: Tuple of emittable signals added automatically

    In addition to the signals defined in :attr:`signals`, all public instance
    variables will have a ``notify::NAME`` signal generated automatically based
    on the ``NAME`` of the variable. ``notify::NAME`` signals will be emitted
    whenever the value of the corresponding instance variable changes.

    Notify signals will be emitted for mutable variables as well, which means
    that care should be taken not to emit thousands of signals when appending
    one-by-one to a large list. :meth:`freeze_notify` and :meth:`thaw_notify`
    will queue notify signals and emit only one of each once thawed.

    The Observable philosophy and API is highly inspired by GObject_.

    .. _GObject: http://www.pygtk.org/docs/pygobject/class-gobject.html
    """

    __metaclass__ = aeidon.Contractual

    __slots__ = ("_blocked_signals",
                 "_blocked_state",
                 "_notify_frozen",
                 "_notify_queue",
                 "_signal_handlers",)

    signals = ()

    def __init__(self):
        """Initialize an :class:`Observable` object."""
        self._blocked_signals = []
        self._blocked_state = False
        self._notify_frozen = False
        self._notify_queue = []
        self._signal_handlers = {}

        for signal in self.signals:
            self._add_signal(signal)

    def __setattr__(self, name, value):
        """Set value of observable attribute."""
        if (name in self.__slots__) or name.startswith("_"):
            return object.__setattr__(self, name, value)
        value = self._validate(name, value)
        signal = "notify::%s" % name
        if not signal in self._signal_handlers:
            self._add_signal(signal)
            return object.__setattr__(self, name, value)
        return_value = object.__setattr__(self, name, value)
        self.emit(signal, value)
        return return_value

    def _add_signal_require(self, signal):
        assert not signal in self._signal_handlers

    def _add_signal(self, signal):
        """Add `signal` to the list of signals emitted."""
        self._signal_handlers[signal] = []

    def _validate_ensure(self, return_value, name, value):
        assert return_value == value

    def _validate(self, name, value):
        """Return `value` or an observable version if `value` is mutable."""
        args = (value, self, name)
        if isinstance(value, dict):
            return aeidon.ObservableDict(*args)
        if isinstance(value, list):
            return aeidon.ObservableList(*args)
        if isinstance(value, set):
            return aeidon.ObservableSet(*args)
        return value

    def block_require(self, signal):
        assert signal in self._signal_handlers

    def block(self, signal):
        """Block all emissions of `signal`.

        Return ``False`` if already blocked, otherwise ``True``.
        """
        if not signal in self._blocked_signals:
            self._blocked_signals.append(signal)
            return True
        return False

    def block_all(self):
        """Block all emissions of all signals.

        Return ``False`` if already blocked, otherwise ``True``.
        """
        if not self._blocked_state:
            self._blocked_state = True
            return True
        return False

    def connect_require(self, signal, method, *args):
        assert signal in self._signal_handlers
        assert callable(method)

    def connect(self, signal, method, *args):
        """Register to receive notifications of ``signal``."""
        self._signal_handlers[signal].append((method, args))

    def disconnect_require(self, signal, method):
        assert signal in self._signal_handlers

    def disconnect(self, signal, method):
        """Remove registration to receive notifications of ``signal``."""
        for i in reversed(range(len(self._signal_handlers[signal]))):
            if self._signal_handlers[signal][i][0] == method:
                self._signal_handlers[signal].pop(i)

    def emit_require(self, signal, *args):
        assert signal in self._signal_handlers

    def emit(self, signal, *args):
        """Send notification of ``signal`` to all registered observers."""
        if signal.startswith("notify::") and self._notify_frozen:
            if not signal in self._notify_queue:
                self._notify_queue.append(signal)
            return

        if ((not self._blocked_state) and
            (not signal in self._blocked_signals)):
            if signal.startswith("notify::"):
                name = signal.replace("notify::", "")
                args = (getattr(self, name),)
            for method, data in self._signal_handlers[signal]:
                method(*((self,) + args + data))

    def freeze_notify(self):
        """Queue notify signals instead of emitting them.

        Return ``False`` if already frozen, otherwise ``True``.
        """
        if not self._notify_frozen:
            self._notify_frozen = True
            return True
        return False

    def notify_require(self, name):
        assert hasattr(self, name)

    def notify(self, name):
        """Emit notification signal for variable."""
        return self.emit("notify::%s" % name)

    def thaw_notify_ensure(self, value, do=True):
        assert (not do) or (not self._notify_queue)

    def thaw_notify(self, do=True):
        """Emit all queued notify signals and queue no more.

        The optional `do` keyword argument should be the return value from
        :meth:`freeze_notify` to avoid problems with nested functions where
        notifications were frozen at a higher level. If `do` is ``False``,
        nothing will be done.

        Return ``False`` if already thawed, otherwise ``True``.
        """
        if do and self._notify_frozen:
            self._notify_frozen = False
            for signal in self._notify_queue:
                name = signal.replace("notify::", "")
                self.emit(signal, getattr(self, name))
            self._notify_queue = []
            return True
        return False

    def unblock_require(self, signal, do=True):
        assert signal in self._signal_handlers

    def unblock(self, signal, do=True):
        """Unblock all emissions of `signal`.

        The optional `do` keyword argument should be the return value from
        :meth:`block` to avoid problems with nested functions where signals
        were blocked at a higher level. If `do` is ``False``, nothing will be
        done.

        Return ``False`` if already unblocked, otherwise ``True``.
        """
        if do and (signal in self._blocked_signals):
            self._blocked_signals.remove(signal)
            return True
        return False

    def unblock_all(self, do=True):
        """Unblock all emissions of all signals.

        The optional `do` keyword argument should be the return value from
        :meth:`block_all` to avoid problems with nested functions where signals
        were blocked at a higher level. If `do` is ``False``, nothing will be
        done.

        Return ``False`` if already unblocked, otherwise ``True``.
        """
        if do and self._blocked_state:
            self._blocked_state = False
            return True
        return False



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Automatic correcting of texts."""

import aeidon
import os
import re
import sys
_ = aeidon.i18n._


class TextAgent(aeidon.Delegate):

    """Automatic correcting of texts."""

    __metaclass__ = aeidon.Contractual
    _re_capitalizable = re.compile(r"^\W*(?<!\.\.\.)\w", re.UNICODE)

    def _capitalize_first(self, parser, pos):
        """Capitalize the first alphanumeric character from `pos`.

        Return ``True`` if something was capitalized, ``False`` if not.
        """
        match = self._re_capitalizable.search(parser.text[pos:])
        if match is not None:
            i = pos + match.end() - 1
            prefix = parser.text[:i]
            text = parser.text[i:i + 1].capitalize()
            suffix = parser.text[i + 1:]
            parser.text = prefix + text + suffix
        return match is not None

    def _capitalize_text(self, parser, pattern, cap_next):
        """Capitalize all matches of `pattern` in `parser`'s text.

        Return ``True`` if the text of the next subtitle should be capitalized.
        """
        try:
            a, z = parser.next()
        except StopIteration:
            return cap_next
        if pattern.get_field("Capitalize") == "Start":
            self._capitalize_first(parser, a)
        if pattern.get_field("Capitalize") == "After":
            cap_next = not self._capitalize_first(parser, z)
        return self._capitalize_text(parser, pattern, cap_next)

    def _get_enchant_checker_require(self, language):
        assert aeidon.util.enchant_available()

    def _get_enchant_checker(self, language):
        """Return an enchant spell-checker for `language`.

        Raise :exc:`enchant.error` if dictionary instatiation fails.
        """
        import enchant.checker
        directory = os.path.join(aeidon.CONFIG_HOME_DIR, "spell-check")
        path = os.path.join(directory, "%s.dict" % language)
        try: dictionary = enchant.DictWithPWL(str(language), str(path))
        except IOError:
            aeidon.util.print_write_io(sys.exc_info(), path)
            dictionary = enchant.Dict(str(language))
        # Sometimes enchant will initialize a dictionary that will not
        # actually work when trying to use it, hence check something.
        dictionary.check("aeidon")
        return enchant.checker.SpellChecker(dictionary, "")

    def _get_misspelled_indices(self, checker):
        """Return a list of misspelled indices in `checker`'s text."""
        i = [range(x.wordpos, x.wordpos + len(x.word)) for x in checker]
        return aeidon.util.flatten(i)

    def _get_substitutions_ensure(self, value, patterns):
        assert len(value) <= len(patterns)

    def _get_substitutions(self, patterns):
        """Return a sequence of tuples of pattern, flags, replacement."""
        return [(x.get_field("Pattern"),
                 x.get_flags(),
                 x.get_field("Replacement")) for x in patterns]

    def _remove_leftover_hi(self, texts, parser):
        """Remove leftover hearing impaired whitespace and junk."""
        texts = texts[:]
        for i, text in enumerate(texts):
            parser.set_text(text)
            # Remove leading and trailing spaces.
            self._replace_all(parser, r"(^\s+|\s+$)", "")
            # Consolidate multiple consequtive spaces.
            self._replace_all(parser, r" {2,}", " ")
            # Remove lines with no alphanumeric characters.
            self._replace_all(parser, r"^\W*$", "")
            # Remove empty lines.
            self._replace_all(parser, r"(^\n|\n$)", "")
            # Add space after dialogue dashes.
            self._replace_all(parser, r"^-(\S)", r"- \1")
            # Remove dialogue dashes if not present on other lines.
            self._replace_all(parser, r"^- (.*?^[^-])", r"\1")
            # Remove dialogue dashes from single-line subtitles.
            self._replace_all(parser, r"\A- ([^\n]*)\Z", r"\1")
            texts[i] = parser.get_text()
        return texts

    def _replace_all(self, parser, pattern, replacement):
        """Replace all matches of `pattern` in `parser`'s text."""
        parser.set_regex(pattern)
        parser.replacement = replacement
        parser.replace_all()

    def break_lines_require(self, indices, *args, **kwargs):
        for index in (indices or ()):
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def break_lines(self,
                    indices,
                    doc,
                    patterns,
                    length_func,
                    max_length,
                    max_lines,
                    max_deviation=None,
                    skip=False,
                    max_skip_length=sys.maxint,
                    max_skip_lines=sys.maxint,
                    register=-1):
        """Break lines to fit defined maximum line length and count.

        `indices` can be ``None`` to process all subtitles. `patterns` should
        be a sequence of instances of :class:`aeidon.Pattern`. `length_func`
        should return the length of a string argument. `max_length` should be
        the maximum allowed length of lines in the same scale as returned by
        `length_func`. `max_lines` may be violated to avoid violating
        `max_length`. `max_deviation` can be used to control the threshold for
        discarding legal solutions for subtitles with three or more lines and a
        variance too great for the subtitle to be typeset elegantly. If `skip`
        is ``True``, subtitles that do not violate or do not manage to reduce
        `max_skip_length` and `max_skip_lines` are skipped.

        Raise :exc:`re.error` if a bad regular expression among `patterns`.
        """
        new_indices = []
        new_texts = []
        patterns = filter(lambda x: x.enabled, patterns)
        patterns = self._get_substitutions(patterns)
        patterns = [(re.compile(x, y), z) for x, y, z in patterns]
        liner = self.get_liner(doc)
        liner.break_points = patterns
        if max_deviation is not None:
            liner.max_deviation = max_deviation
        liner.max_length = max_length
        liner.max_lines = max_lines
        liner.length_func = length_func
        re_tag = self.get_markup_tag_regex(doc)
        for index in indices or self.get_all_indices():
            subtitle = self.subtitles[index]
            liner.set_text(subtitle.get_text(doc))
            plain_text = subtitle.get_text(doc)
            if re_tag is not None:
                plain_text = re_tag.sub("", plain_text)
            lines = plain_text.split("\n")
            length = max(map(length_func, lines))
            line_count = len(lines)
            if length <= max_skip_length:
                if line_count <= max_skip_lines:
                    # Skip subtitles that do not violate
                    # any of the defined skip conditions.
                    if skip: continue
            text = liner.break_lines()
            if re_tag is not None:
                plain_text = re_tag.sub("", text)
            lines = plain_text.split("\n")
            length_down = max(map(length_func, lines)) < length
            lines_down = len(lines) < line_count
            length_fixed = (length > max_skip_length) and length_down
            lines_fixed = (line_count > max_skip_lines) and lines_down
            if (not length_fixed) and (not lines_fixed):
                # Implicitly require reduction of violator
                # if only lines in violation are to be broken.
                if skip: continue
            if text != subtitle.get_text(doc):
                new_indices.append(index)
                new_texts.append(text)
        if not new_indices: return
        self.replace_texts(new_indices, doc, new_texts, register=register)
        self.set_action_description(register, _("Breaking lines"))

    def capitalize_require(self, indices, doc, pattern, register=-1):
        for index in (indices or ()):
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def capitalize(self, indices, doc, patterns, register=-1):
        """Capitalize texts as defined by `patterns`.

        `indices` can be ``None`` to process all subtitles. `patterns` should
        be a sequence of instances of :class:`aeidon.Pattern`. Raise
        :exc:`re.error` if a bad regular expression among `patterns`.
        """
        new_indices = []
        new_texts = []
        parser = self.get_parser(doc)
        patterns = filter(lambda x: x.enabled, patterns)
        indices = indices or self.get_all_indices()
        for indices in aeidon.util.get_ranges(indices):
            cap_next = False
            for index in indices:
                subtitle = self.subtitles[index]
                parser.set_text(subtitle.get_text(doc))
                if cap_next or (index == 0):
                    self._capitalize_first(parser, 0)
                    cap_next = False
                for pattern in patterns:
                    string = pattern.get_field("Pattern")
                    flags = pattern.get_flags()
                    parser.set_regex(string, flags, 0)
                    parser.pos = 0
                    cap_next = self._capitalize_text(parser,
                                                     pattern,
                                                     cap_next)

                text = parser.get_text()
                if text != subtitle.get_text(doc):
                    new_indices.append(index)
                    new_texts.append(text)
        if not new_indices: return
        self.replace_texts(new_indices, doc, new_texts, register=register)
        self.set_action_description(register, _("Capitalizing texts"))

    def correct_common_errors_require(self, indices, *args, **kwargs):
        for index in (indices or ()):
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def correct_common_errors(self, indices, doc, patterns, register=-1):
        """Correct common human and OCR errors in texts.

        `indices` can be ``None`` to process all subtitles. `patterns` should
        be a sequence of instances of :class:`aeidon.Pattern`. Raise
        :exc:`re.error` if a bad regular expression among `patterns`.
        """
        new_indices = []
        new_texts = []
        parser = self.get_parser(doc)
        patterns = filter(lambda x: x.enabled, patterns)
        re_patterns = self._get_substitutions(patterns)
        repeats = [x.get_field_boolean("Repeat") for x in patterns]
        for index in indices or self.get_all_indices():
            subtitle = self.subtitles[index]
            parser.set_text(subtitle.get_text(doc))
            for i, (string, flags, replacement) in enumerate(re_patterns):
                parser.set_regex(string, flags, 0)
                parser.replacement = replacement
                count = parser.replace_all()
                while repeats[i] and count:
                    count = parser.replace_all()
            text = parser.get_text()
            if text != subtitle.get_text(doc):
                new_indices.append(index)
                new_texts.append(text)
        if not new_indices: return
        self.replace_texts(new_indices, doc, new_texts, register=register)
        self.set_action_description(register, _("Correcting common errors"))

    def remove_hearing_impaired_require(self, indices, *args, **kwargs):
        for index in (indices or ()):
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def remove_hearing_impaired(self, indices, doc, patterns, register=-1):
        """Remove hearing impaired parts from subtitles.

        `indices` can be ``None`` to process all subtitles. `patterns` should
        be a sequence of instances of :class:`aeidon.Pattern`. Raise
        :exc:`re.error` if a bad regular expression among `patterns`.
        """
        new_indices = []
        new_texts = []
        parser = self.get_parser(doc)
        patterns = filter(lambda x: x.enabled, patterns)
        re_patterns = self._get_substitutions(patterns)
        for index in indices or self.get_all_indices():
            subtitle = self.subtitles[index]
            parser.set_text(subtitle.get_text(doc))
            for string, flags, replacement in re_patterns:
                parser.set_regex(string, flags, 0)
                parser.replacement = replacement
                parser.replace_all()
            text = parser.get_text()
            if text != subtitle.get_text(doc):
                new_indices.append(index)
                new_texts.append(text)
        if not new_indices: return
        new_texts = self._remove_leftover_hi(new_texts, parser)
        self.replace_texts(new_indices, doc, new_texts, register=register)
        description = _("Removing hearing impaired texts")
        self.set_action_description(register, description)
        remove_indices = []
        for i, text in (x for x in enumerate(new_texts) if not x[1]):
            remove_indices.append(new_indices[i])
        if not remove_indices: return
        self.remove_subtitles(remove_indices, register=register)
        self.group_actions(register, 2, description)

    def spell_check_join_words_require(self, indices, *args, **kwargs):
        for index in (indices or ()):
            assert 0 <= index < len(self.subtitles)
        assert aeidon.util.enchant_available()

    @aeidon.deco.export
    @aeidon.deco.revertable
    def spell_check_join_words(self, indices, doc, language, register=-1):
        """Join misspelled words based on spell-checker suggestions.

        Raise :exc:`enchant.Error` if dictionary instatiation fails.
        """
        new_indices = []
        new_texts = []
        re_multispace = re.compile(r" +")
        checker = self._get_enchant_checker(language)
        seeker = self._get_enchant_checker(language)
        for index in indices or self.get_all_indices():
            subtitle = self.subtitles[index]
            text = subtitle.get_text(doc)
            text = re_multispace.sub(" ", text)
            checker.set_text(unicode(text))
            while True:
                try: checker.next()
                except StopIteration: break
                text = checker.get_text()
                a = checker.wordpos
                z = checker.wordpos + len(checker.word)
                ok_with_prev = ok_with_next = False
                if checker.leading_context(1) == " ":
                    seeker.set_text(text[:a - 1] + text[a:])
                    poss = self._get_misspelled_indices(seeker)
                    ok_with_prev = not (a - 1) in poss
                if checker.trailing_context(1) == " ":
                    seeker.set_text(text[:z] + text[z + 1:])
                    poss = self._get_misspelled_indices(seeker)
                    ok_with_next = not a in poss
                # Join backwards or forwards if only one direction,
                # but not both, produce a correctly spelled result.
                if ok_with_prev and not ok_with_next:
                    checker.set_text(text[:a - 1] + text[a:])
                if ok_with_next and not ok_with_prev:
                    checker.set_text(text[:z] + text[z + 1:])
            new_text = unicode(checker.get_text())
            if new_text != text:
                new_indices.append(index)
                new_texts.append(new_text)
        if not new_indices: return
        self.replace_texts(new_indices, doc, new_texts, register=register)
        description = _("Joining words by spell-check suggestions")
        self.set_action_description(register, description)

    def spell_check_split_words_require(self, indices, *args, **kwargs):
        for index in (indices or ()):
            assert 0 <= index < len(self.subtitles)
        assert aeidon.util.enchant_available()

    @aeidon.deco.export
    @aeidon.deco.revertable
    def spell_check_split_words(self, indices, doc, language, register=-1):
        """Split misspelled words based on spell-checker suggestions.

        Using this is usually not a good idea unless you have an insane
        dictionary that contains all possible compound words in `language`.
        Raise :exc:`enchant.Error` if dictionary instatiation fails.
        """
        new_indices = []
        new_texts = []
        re_multispace = re.compile(r" +")
        checker = self._get_enchant_checker(language)
        indices = indices or self.get_all_indices()
        for index in indices:
            subtitle = self.subtitles[index]
            text = subtitle.get_text(doc)
            text = re_multispace.sub(" ", text)
            checker.set_text(unicode(text))
            while True:
                try: checker.next()
                except StopIteration: break
                if checker.word.capitalize() == checker.word:
                    # Skip capitalized words, which are usually names
                    # and thus not always found in dictionaries.
                    continue
                length = len(checker.word)
                suggestions = []
                for i, suggestion in enumerate(checker.suggest()):
                    if suggestion.find(" ") > 0:
                        if suggestion.replace(" ", "") == checker.word:
                            suggestions.append(suggestion)
                # Split word only if only one two-word suggestion found that
                # has all the same characters as the original unsplit word.
                if len(suggestions) != 1: continue
                text = checker.get_text()
                a = checker.wordpos
                z = checker.wordpos + len(checker.word)
                checker.set_text(text[:a] + suggestions[0] + text[z:])
            new_text = unicode(checker.get_text())
            if new_text != text:
                new_indices.append(index)
                new_texts.append(new_text)
        if not new_indices: return
        self.replace_texts(new_indices, doc, new_texts, register=register)
        description = _("Splitting words by spell-check suggestions")
        self.set_action_description(register, description)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import functools
import gaupol
import gtk
import os


def while_errors(function):
    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        while True:
            value = function(*args, **kwargs)
            # Break when no more errors left.
            if not args[0].dialog._table.props.sensitive: break
        return value
    return wrapper


class TestSpellCheckDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def run__show_error_dialog(self):
        self.dialog._show_error_dialog("test")

    def setup_method(self, method):
        gaupol.conf.editor.use_custom_font = True
        gaupol.conf.editor.custom_font = "monospace"
        gaupol.conf.spell_check.language = "en"
        self.application = self.new_application()
        for page in self.application.pages:
            for subtitle in page.project.subtitles:
                subtitle.main_text = subtitle.main_text.replace("a", "x")
                subtitle.tran_text = subtitle.tran_text.replace("a", "x")
            page.reload_view_all()
        self.dialog = gaupol.SpellCheckDialog(self.application.window,
                                              self.application)

        # Avoid adding words to either enchant's or a backend's
        # personal word list or gaupol's personal replacement list.
        self.dialog._checker.dict.add = lambda *args: None
        self._temp_dir = aeidon.temp.create_directory()
        self.dialog._personal_dir = self._temp_dir
        self.dialog.show()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test___init____enchant_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        gaupol.conf.spell_check.language = "wo"
        self.raises(ValueError,
                    gaupol.SpellCheckDialog,
                    self.application.window,
                    self.application)

    def test___init____replacements(self):
        basename = "%s.repl" % gaupol.conf.spell_check.language
        path = os.path.join(self.dialog._personal_dir, basename)
        open(path, "w").write("a|b\nc|d\n")
        gaupol.SpellCheckDialog(self.application.window,
                                self.application)

    @while_errors
    def test__on_add_button_clicked(self):
        self.dialog._checker.dict.add = lambda *args: None
        self.dialog._add_button.emit("clicked")

    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_edit_button_clicked(self):
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._edit_button.emit("clicked")

    def test__on_entry_changed(self):
        self.dialog._entry.set_text("t")
        self.dialog._entry.set_text("te")
        self.dialog._entry.set_text("tes")
        self.dialog._entry.set_text("test")

    @while_errors
    def test__on_ignore_all_button_clicked(self):
        self.dialog._ignore_all_button.emit("clicked")

    @while_errors
    def test__on_ignore_button_clicked(self):
        self.dialog._ignore_button.emit("clicked")

    @while_errors
    def test__on_join_back_button_clicked(self):
        if self.dialog._join_back_button.props.sensitive:
            self.dialog._join_back_button.emit("clicked")
        else: self.dialog._ignore_button.emit("clicked")

    @while_errors
    def test__on_join_forward_button_clicked(self):
        if self.dialog._join_forward_button.props.sensitive:
            self.dialog._join_forward_button.emit("clicked")
        else: self.dialog._ignore_button.emit("clicked")

    @while_errors
    def test__on_replace_all_button_clicked(self):
        self.dialog._replace_all_button.emit("clicked")

    @while_errors
    def test__on_replace_button_clicked(self):
        self.dialog._replace_button.emit("clicked")

    def test__on_response(self):
        self.dialog._replace_button.emit("clicked")
        self.dialog.response(gtk.RESPONSE_CLOSE)

    def test__on_tree_view_selection_changed(self):
        store = self.dialog._tree_view.get_model()
        selection = self.dialog._tree_view.get_selection()
        for i in range(len(store)):
            selection.select_path(i)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._show_error_dialog("test")



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestCalculator(aeidon.TestCase):

    def setup_method(self, method):
        self.calc = aeidon.Calculator(aeidon.framerates.FPS_23_976)

    def test___new__(self):
        for framerate in aeidon.framerates:
            a = aeidon.Calculator(framerate)
            b = aeidon.Calculator(framerate)
            assert a is b

    def test_add_seconds_to_time(self):
        add_seconds = self.calc.add_seconds_to_time
        assert add_seconds( "00:00:00.001",  5) ==  "00:00:05.001"
        assert add_seconds( "00:00:00.001", -5) == "-00:00:04.999"
        assert add_seconds("-00:00:00.001",  5) ==  "00:00:04.999"
        assert add_seconds("-00:00:00.001", -5) == "-00:00:05.001"

    def test_add_times(self):
        for x, y, z in (( "33:33:33.333",  "44:44:44.444",  "78:18:17.777"),
                        ("-33:33:33.333", "-44:44:44.444", "-78:18:17.777"),
                        ("-33:33:33.333",  "44:44:44.444",  "11:11:11.111"),
                        ( "33:33:33.333", "-44:44:44.444", "-11:11:11.111")):
            assert self.calc.add_times(x, y) == z

    def test_compare(self):
        compare = self.calc.compare
        assert compare("00:00:00.001", "00:00:00.000") ==  1
        assert compare( 1,  0) ==  1
        assert compare( 0,  0) ==  0
        assert compare( 0,  1) == -1
        assert compare(-2, -1) == -1
        assert compare(-1, -1) ==  0
        assert compare(-1, -2) ==  1
        assert compare( 1, -1) ==  1
        assert compare(-1,  1) == -1

    def test_compare__value_error(self):
        self.raises(ValueError, self.calc.compare, None, None)

    def test_compare_times(self):
        compare = self.calc.compare_times
        assert compare( "00:00:00.001",  "00:00:00.000") ==  1
        assert compare( "00:00:00.000",  "00:00:00.000") ==  0
        assert compare( "00:00:00.000",  "00:00:00.001") == -1
        assert compare("-00:00:00.002", "-00:00:00.001") == -1
        assert compare("-00:00:00.001", "-00:00:00.001") ==  0
        assert compare("-00:00:00.001", "-00:00:00.002") ==  1
        assert compare( "00:00:00.001", "-00:00:00.001") ==  1
        assert compare("-00:00:00.001",  "00:00:00.001") == -1

    def test_frame_to_seconds(self):
        calc = aeidon.Calculator(aeidon.framerates.FPS_25_000)
        assert calc.frame_to_seconds( 127) ==  5.08
        assert calc.frame_to_seconds(-127) == -5.08

    def test_frame_to_time(self):
        assert self.calc.frame_to_time( 2658) ==  "00:01:50.861"
        assert self.calc.frame_to_time(-2658) == "-00:01:50.861"

    def test_get_frame_duration(self):
        assert self.calc.get_frame_duration( 561,  1048) ==  487
        assert self.calc.get_frame_duration( 561,   560) ==   -1
        assert self.calc.get_frame_duration(-561, -1048) == -487
        assert self.calc.get_frame_duration(-561,  -560) ==    1

    def test_get_middle__frame(self):
        assert self.calc.get_middle( 300,  400) ==  350
        assert self.calc.get_middle( 300,  401) ==  351
        assert self.calc.get_middle(-300, -400) == -350
        assert self.calc.get_middle(-300,  401) ==   51

    def test_get_middle__time(self):
        for x, y, z in (( "00:00:01.000",  "00:00:02.000",  "00:00:01.500"),
                        ( "00:00:01.000",  "00:00:02.001",  "00:00:01.501"),
                        ("-00:00:01.000", "-00:00:02.000", "-00:00:01.500"),
                        ( "00:00:01.000", "-00:00:02.002", "-00:00:00.501")):
            assert self.calc.get_middle(x, y) == z

    def test_get_middle__value_error(self):
        self.raises(ValueError, self.calc.get_middle, None, None)

    def test_get_time_duration(self):
        for x, y, z in (( "00:01:22.500",  "00:01:45.100",  "00:00:22.600"),
                        ( "00:01:22.500",  "00:01:00.100", "-00:00:22.400"),
                        ("-00:01:22.500",  "00:01:45.100",  "00:03:07.600"),
                        ( "00:01:22.500", "-00:01:00.100", "-00:02:22.600")):
            assert self.calc.get_time_duration(x, y) == z

    def test_is_valid_time__false(self):
        assert not self.calc.is_valid_time("2:34:56.789")
        assert not self.calc.is_valid_time("12:60:56.789")

    def test_is_valid_time__true(self):
        assert self.calc.is_valid_time("12:34:56.789")
        assert self.calc.is_valid_time("-12:34:56.789")

    def test_parse_time(self):
        assert self.calc.parse_time("1:2:3.4") == "01:02:03.400"
        assert self.calc.parse_time("1:2:3,4") == "01:02:03.400"
        assert self.calc.parse_time("-00:00:00.400") == "-00:00:00.400"
        assert self.calc.parse_time("-01:02:03.400") == "-01:02:03.400"

    def test_round_time(self):
        round_time = self.calc.round_time
        assert round_time("02:36:35.857", 3) == "02:36:35.857"
        assert round_time("02:36:35.857", 2) == "02:36:35.860"
        assert round_time("02:36:35.857", 1) == "02:36:35.900"
        assert round_time("02:36:35.857", 0) == "02:36:36.000"

    def test_seconds_to_frame(self):
        seconds_to_frame = self.calc.seconds_to_frame
        assert seconds_to_frame(6552) == 157091
        assert seconds_to_frame(-337) ==  -8080

    def test_seconds_to_time(self):
        seconds_to_time = self.calc.seconds_to_time
        assert seconds_to_time(68951.15388) ==  "19:09:11.154"
        assert seconds_to_time(-12.0000000) == "-00:00:12.000"
        assert seconds_to_time(999999.0000) ==  "99:59:59.999"

    def test_time_to_frame(self):
        time_to_frame = self.calc.time_to_frame
        assert time_to_frame( "01:22:36.144") == 118829
        assert time_to_frame("-00:13:43.087") == -19734

    def test_time_to_seconds(self):
        time_to_seconds = self.calc.time_to_seconds
        assert time_to_seconds( "03:45:22.117") == 13522.117
        assert time_to_seconds("-00:00:45.000") ==   -45.000

    def test_to_frame(self):
        self.calc = aeidon.Calculator(aeidon.framerates.FPS_25_000)
        assert self.calc.to_frame("00:00:01.000") == 25
        assert self.calc.to_frame(25) == 25
        assert self.calc.to_frame(1.0) == 25

    def test_to_frame__value_error(self):
        self.raises(ValueError, self.calc.to_frame, None)

    def test_to_seconds(self):
        self.calc = aeidon.Calculator(aeidon.framerates.FPS_25_000)
        assert self.calc.to_seconds("00:00:01.000") == 1.0
        assert self.calc.to_seconds(25) == 1.0
        assert self.calc.to_seconds(1.0) == 1.0

    def test_to_seconds__value_error(self):
        self.raises(ValueError, self.calc.to_seconds, None)

    def test_to_time(self):
        self.calc = aeidon.Calculator(aeidon.framerates.FPS_25_000)
        assert self.calc.to_time("00:00:01.000") == "00:00:01.000"
        assert self.calc.to_time(25) == "00:00:01.000"
        assert self.calc.to_time(1.0) == "00:00:01.000"

    def test_to_time__value_error(self):
        self.raises(ValueError, self.calc.to_time, None)



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestPatternManager(aeidon.TestCase):

    def setup_method(self, method):
        self.manager = aeidon.PatternManager("common-error")

    def test_get_countries(self):
        countries = self.manager.get_countries("Latn", "en")
        assert "US" in countries

    def test_get_languages(self):
        languages = self.manager.get_languages("Latn")
        assert "en" in languages

    def test_get_patterns(self):
        assert self.manager.get_patterns("Latn")
        assert self.manager.get_patterns("Latn", "en")
        assert self.manager.get_patterns("Latn", "en", "US")

    def test_get_patterns__policy_replace(self):
        self.manager = aeidon.PatternManager("line-break")
        assert self.manager.get_patterns("Latn", "en")

    def test_get_scripts(self):
        scripts = self.manager.get_scripts()
        assert "Latn" in scripts

    @aeidon.deco.monkey_patch(aeidon, "CONFIG_HOME_DIR")
    def test_save_config(self):
        aeidon.CONFIG_HOME_DIR = aeidon.temp.create_directory()
        self.manager.save_config("Latn")
        self.manager._read_patterns()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import codecs
_ = aeidon.i18n._


class TestModule(aeidon.TestCase):

    def test_code_to_description(self):
        code_to_description = aeidon.encodings.code_to_description
        assert code_to_description("cp1006") == _("Urdu")
        assert code_to_description("hz") == _("Chinese simplified")
        assert code_to_description("shift_jis") == _("Japanese")

    def test_code_to_description__value_error(self):
        code_to_description = aeidon.encodings.code_to_description
        self.raises(ValueError, code_to_description, "xxxxx")

    def test_code_to_long_name(self):
        code, name, description = ("cp1140", "IBM1140", _("Western"))
        long_name = aeidon.encodings.code_to_long_name(code)
        assert long_name == _("%(description)s (%(name)s)") % locals()

    def test_code_to_long_name__value_error(self):
        self.raises(ValueError, aeidon.encodings.code_to_long_name, "xxxxx")

    def test_code_to_name(self):
        code_to_name = aeidon.encodings.code_to_name
        assert code_to_name("big5hkscs") == "Big5-HKSCS"
        assert code_to_name("cp949") == "IBM949"
        assert code_to_name("mac_roman") == "MacRoman"

    def test_code_to_name__value_error(self):
        code_to_name = aeidon.encodings.code_to_name
        self.raises(ValueError, code_to_name, "xxxxx")

    def test_detect(self):
        name = aeidon.encodings.detect(self.new_subrip_file())
        assert aeidon.encodings.is_valid_code(name)

    @aeidon.deco.monkey_patch(aeidon.encodings, "translate_code")
    def test_detect__value_error(self):
        def bad_translate_code(code): raise ValueError
        aeidon.encodings.translate_code = bad_translate_code
        assert aeidon.encodings.detect(self.new_subrip_file()) is None

    def test_detect_bom__none(self):
        path = self.new_subrip_file()
        encoding = aeidon.encodings.detect_bom(path)
        assert encoding is None

    def test_detect_bom__utf_8(self):
        path = self.new_subrip_file()
        text = open(path, "r").read()
        open(path, "w").write(codecs.BOM_UTF8 + text)
        encoding = aeidon.encodings.detect_bom(path)
        if aeidon.encodings.is_valid_code("utf_8_sig"):
            assert encoding == "utf_8_sig"

    def test_detect_bom__utf_16_be(self):
        path = self.new_subrip_file()
        text = open(path, "r").read()
        open(path, "w").write(codecs.BOM_UTF16_BE + text)
        encoding = aeidon.encodings.detect_bom(path)
        if aeidon.encodings.is_valid_code("utf_16_be"):
            assert encoding == "utf_16_be"

    def test_detect_bom__utf_16_le(self):
        path = self.new_subrip_file()
        text = open(path, "r").read()
        open(path, "w").write(codecs.BOM_UTF16_LE + text)
        encoding = aeidon.encodings.detect_bom(path)
        if aeidon.encodings.is_valid_code("utf_16_le"):
            assert encoding == "utf_16_le"

    def test_detect_bom__utf_32_be(self):
        path = self.new_subrip_file()
        text = open(path, "r").read()
        open(path, "w").write(codecs.BOM_UTF32_BE + text)
        encoding = aeidon.encodings.detect_bom(path)
        if aeidon.encodings.is_valid_code("utf_32_be"):
            assert encoding == "utf_32_be"

    def test_detect_bom__utf_32_le(self):
        path = self.new_subrip_file()
        text = open(path, "r").read()
        open(path, "w").write(codecs.BOM_UTF32_LE + text)
        encoding = aeidon.encodings.detect_bom(path)
        if aeidon.encodings.is_valid_code("utf_32_le"):
            assert encoding == "utf_32_le"

    def test_get_locale_code(self):
        code = aeidon.encodings.get_locale_code()
        assert aeidon.encodings.is_valid_code(code)

    def test_get_locale_long_name(self):
        long_name = aeidon.encodings.get_locale_long_name()
        code = aeidon.encodings.get_locale_code()
        name = aeidon.encodings.code_to_name(code)
        assert long_name == _("Current locale (%s)") % name

    def test_get_valid(self):
        assert aeidon.encodings.get_valid()
        for item in aeidon.encodings.get_valid():
            assert aeidon.encodings.is_valid_code(item[0])
            assert isinstance(item[1], basestring)
            assert isinstance(item[2], basestring)

    @aeidon.deco.monkey_patch(aeidon.encodings, "is_valid_code")
    def test_get_valid__invalid(self):
        bad_is_valid_code = lambda code: not code.startswith("cp")
        aeidon.encodings.is_valid_code = bad_is_valid_code
        assert aeidon.encodings.get_valid()

    def test_is_valid_code(self):
        assert aeidon.encodings.is_valid_code("gbk")
        assert aeidon.encodings.is_valid_code("utf_16_be")
        assert not aeidon.encodings.is_valid_code("xxxxx")

    def test_name_to_code(self):
        name_to_code = aeidon.encodings.name_to_code
        assert name_to_code("IBM037") == "cp037"
        assert name_to_code("GB2312") == "gb2312"
        assert name_to_code("PTCP154") == "ptcp154"

    def test_name_to_code__value_error(self):
        name_to_code = aeidon.encodings.name_to_code
        self.raises(ValueError, name_to_code, "XXXXX")

    def test_translate_code(self):
        translate_code = aeidon.encodings.translate_code
        assert translate_code("johab") == "johab"
        assert translate_code("UTF-8") == "utf_8"
        assert translate_code("ISO-8859-1") == "latin_1"

    def test_translate_code__value_error(self):
        translate_code = aeidon.encodings.translate_code
        self.raises(ValueError, translate_code, "xxxxx")



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import re


class TestFinder(aeidon.TestCase):

    text = ("One only risks it, because\n"
            "one's survival depends on it.")

    def assert_find_cases(self, cases, regex, next):
        advance = (self.finder.next if next else self.finder.previous)
        for pattern, matches in cases:
            self.finder.set_text(self.text, next)
            if regex:
                self.finder.set_regex(pattern)
            else: # String pattern.
                self.finder.pattern = pattern
            for match in matches:
                assert advance() == match
                assert self.finder.pos == match[next]
            self.raises(StopIteration, advance)

    def assert_replace_all_cases(self, cases, regex):
        for pattern, replacement, count, text in cases:
            self.finder.set_text(self.text)
            if regex:
                self.finder.set_regex(pattern)
            else: # String pattern.
                self.finder.pattern = pattern
            self.finder.replacement = replacement
            assert self.finder.replace_all() == count
            assert self.finder.text == text

    def setup_method(self, method):
        self.finder = aeidon.Finder()

    def test_next__regex(self):
        cases = ((r"a", (
                    (22, 23),
                    (39, 40),)),
                 (r"it", (
                    (15, 17),
                    (53, 55),)),
                 (r"^", (
                    ( 0,  0),
                    (27, 27),)),
                 (r"\A", (
                    ( 0,  0),)),
                 (r"$", (
                    (26, 26),
                    (56, 56),)),
                 (r"\Z", (
                    (56, 56),)),
                 (r"\b", (
                    ( 0,  0),
                    ( 3,  3),
                    ( 4,  4),
                    ( 8,  8),
                    ( 9,  9),
                    (14, 14),
                    (15, 15),
                    (17, 17),
                    (19, 19),
                    (26, 26),
                    (27, 27),
                    (30, 30),
                    (31, 31),
                    (32, 32),
                    (33, 33),
                    (41, 41),
                    (42, 42),
                    (49, 49),
                    (50, 50),
                    (52, 52),
                    (53, 53),
                    (55, 55),)),
                 (r"\s", (
                    ( 3,  4),
                    ( 8,  9),
                    (14, 15),
                    (18, 19),
                    (26, 27),
                    (32, 33),
                    (41, 42),
                    (49, 50),
                    (52, 53),)),
                 (r"\w+", (
                    ( 0,  3),
                    ( 4,  8),
                    ( 9, 14),
                    (15, 17),
                    (19, 26),
                    (27, 30),
                    (31, 32),
                    (33, 41),
                    (42, 49),
                    (50, 52),
                    (53, 55),)),
                 (r"\W{2}", (
                    (17, 19),)))

        self.assert_find_cases(cases, True, True)

    def test_next__regex_ignore_case(self):
        cases = ((r"O" , (
                    ( 0,  1),
                    ( 4,  5),
                    (27, 28),
                    (50, 51),)),
                 (r"D", (
                    (42, 43),
                    (47, 48),)))

        self.finder.ignore_case = True
        self.assert_find_cases(cases, True, True)

    def test_next__string(self):
        cases = (("a" , (
                    (22, 23),
                    (39, 40),)),
                 ("it", (
                    (15, 17),
                    (53, 55),)),
                 ("O" , (
                    ( 0,  1),)),
                 (" ", (
                    ( 3,  4),
                    ( 8,  9),
                    (14, 15),
                    (18, 19),
                    (32, 33),
                    (41, 42),
                    (49, 50),
                    (52, 53),)),
                 ("\n", (
                    (26, 27),)),
                 ("." , (
                    (55, 56),)))

        self.assert_find_cases(cases, False, True)

    def test_next__string_ignore_case(self):
        cases = (("o" , (
                    ( 0,  1),
                    ( 4,  5),
                    (27, 28),
                    (50, 51),)),
                 ("k", (
                    (12, 13),)))

        self.finder.ignore_case = True
        self.assert_find_cases(cases, False, True)

    def test_previous__regex(self):
        cases = ((r"a" , (
                    (39, 40),
                    (22, 23),)),
                 (r"it", (
                    (53, 55),
                    (15, 17),)),
                 (r"^", (
                    (27, 27),
                    ( 0,  0),)),
                 (r"\A", (
                    ( 0,  0),)),
                 (r"$", (
                    (56, 56),
                    (26, 26),)),
                 (r"\Z", (
                    (56, 56),)),
                 (r"\b", (
                    (55, 55),
                    (53, 53),
                    (52, 52),
                    (50, 50),
                    (49, 49),
                    (42, 42),
                    (41, 41),
                    (33, 33),
                    (32, 32),
                    (31, 31),
                    (30, 30),
                    (27, 27),
                    (26, 26),
                    (19, 19),
                    (17, 17),
                    (15, 15),
                    (14, 14),
                    ( 9,  9),
                    ( 8,  8),
                    ( 4,  4),
                    ( 3,  3),
                    ( 0,  0),)),
                 (r"\s", (
                    (52, 53),
                    (49, 50),
                    (41, 42),
                    (32, 33),
                    (26, 27),
                    (18, 19),
                    (14, 15),
                    ( 8,  9),
                    ( 3,  4),)),
                 (r"\w+", (
                    (53, 55),
                    (50, 52),
                    (42, 49),
                    (33, 41),
                    (31, 32),
                    (27, 30),
                    (19, 26),
                    (15, 17),
                    ( 9, 14),
                    ( 4,  8),
                    ( 0,  3),)),
                 (r"\W{2}", (
                    (17, 19),)))

        self.assert_find_cases(cases, True, False)

    def test_previous__regex_ignore_case(self):
        cases = ((r"O" , (
                    (50, 51),
                    (27, 28),
                    ( 4,  5),
                    ( 0,  1),)),
                 (r"D", (
                    (47, 48),
                    (42, 43),)))

        self.finder.ignore_case = True
        self.assert_find_cases(cases, True, False)

    def test_previous__string(self):
        cases = (("a" , (
                    (39, 40),
                    (22, 23),)),
                 ("it", (
                    (53, 55),
                    (15, 17),)),
                 ("O" , (
                    (0, 1),)),
                 (" ", (
                    (52, 53),
                    (49, 50),
                    (41, 42),
                    (32, 33),
                    (18, 19),
                    (14, 15),
                    ( 8,  9),
                    ( 3,  4),)),
                 ("\n", (
                    (26, 27),)),
                 ("." , (
                    (55, 56),)))

        self.assert_find_cases(cases, False, False)

    def test_previous__string_ignore_case(self):
        cases = (("o" , (
                    (50, 51),
                    (27, 28),
                    ( 4,  5),
                    ( 0,  1),)),
                 ("k", (
                    (12, 13),)))

        self.finder.ignore_case = True
        self.assert_find_cases(cases, False, False)

    def test_replace__equal_length_next(self):
        self.finder.set_text(self.text)
        self.finder.pattern = "ne"
        self.finder.replacement = "--"
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "O-- only risks it, because\n"
            "one's survival depends on it.")
        assert self.finder.pos == 3
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "O-- only risks it, because\n"
            "o--'s survival depends on it.")
        assert self.finder.pos == 30
        self.raises(StopIteration, self.finder.next)

    def test_replace__equal_length_previous(self):
        self.finder.set_text(self.text, False)
        self.finder.pattern = "ne"
        self.finder.replacement = "--"
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "o--'s survival depends on it.")
        assert self.finder.pos == 28
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "O-- only risks it, because\n"
            "o--'s survival depends on it.")
        assert self.finder.pos == 1
        self.raises(StopIteration, self.finder.previous)

    def test_replace__lengthen_regex_dollar_next(self):
        self.finder.set_text(self.text)
        self.finder.set_regex(r"$")
        self.finder.replacement = "--"
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it, because--\n"
            "one's survival depends on it.")
        assert self.finder.pos == 28
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it, because--\n"
            "one's survival depends on it.--")
        assert self.finder.pos == 60
        self.raises(StopIteration, self.finder.next)

    def test_replace__lengthen_regex_dollar_previous(self):
        self.finder.set_text(self.text, False)
        self.finder.set_regex(r"$")
        self.finder.replacement = "--"
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survival depends on it.--")
        assert self.finder.pos == 56
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it, because--\n"
            "one's survival depends on it.--")
        assert self.finder.pos == 26
        self.raises(StopIteration, self.finder.previous)

    def test_replace__lengthen_regex_hat_next(self):
        self.finder.set_text(self.text)
        self.finder.set_regex(r"^")
        self.finder.replacement = "--"
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "--One only risks it, because\n"
            "one's survival depends on it.")
        assert self.finder.pos == 2
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "--One only risks it, because\n"
            "--one's survival depends on it.")
        assert self.finder.pos == 31
        self.raises(StopIteration, self.finder.next)

    def test_replace__lengthen_regex_hat_previous(self):
        self.finder.set_text(self.text, False)
        self.finder.set_regex(r"^")
        self.finder.replacement = "--"
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "--one's survival depends on it.")
        assert self.finder.pos == 27
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "--One only risks it, because\n"
            "--one's survival depends on it.")
        assert self.finder.pos == 0
        self.raises(StopIteration, self.finder.previous)

    def test_replace__lengthen_string_match_next(self):
        self.finder.set_text(self.text)
        self.finder.pattern = "v"
        self.finder.replacement = "vv"
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survvival depends on it.")
        assert self.finder.pos == 38
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survvivval depends on it.")
        assert self.finder.pos == 41
        self.raises(StopIteration, self.finder.next)

    def test_replace__lengthen_string_match_previous(self):
        self.finder.set_text(self.text, False)
        self.finder.pattern = "v"
        self.finder.replacement = "vv"
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survivval depends on it.")
        assert self.finder.pos == 38
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survvivval depends on it.")
        assert self.finder.pos == 36
        self.raises(StopIteration, self.finder.previous)

    def test_replace__shorten_regex_next(self):
        self.finder.set_text(self.text)
        self.finder.set_regex(r"[.,]")
        self.finder.replacement = ""
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it because\n"
            "one's survival depends on it.")
        assert self.finder.pos == 17
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it because\n"
            "one's survival depends on it")
        assert self.finder.pos == 54
        self.raises(StopIteration, self.finder.next)

    def test_replace__shorten_regex_previous(self):
        self.finder.set_text(self.text, False)
        self.finder.set_regex(r"[.,]")
        self.finder.replacement = ""
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survival depends on it")
        assert self.finder.pos == 55
        self.finder.previous()
        self.finder.replace(False)
        assert self.finder.text == (
            "One only risks it because\n"
            "one's survival depends on it")
        assert self.finder.pos == 17
        self.raises(StopIteration, self.finder.previous)

    def test_replace_all__regex(self):
        cases = ((r"e", r"-", 6,
                  ("On- only risks it, b-caus-\n"
                   "on-'s survival d-p-nds on it.")),
                 (r"\b", r"-", 22,
                  ("-One- -only- -risks- -it-, -because-\n"
                   "-one-'-s- -survival- -depends- -on- -it-.")),
                 (r"\s", r"", 9,
                  ("Oneonlyrisksit,because"
                   "one'ssurvivaldependsonit.")),
                 (r"^", r"-", 2,
                  ("-One only risks it, because\n"
                   "-one's survival depends on it.")),
                 (r"\A", r"-", 1,
                  ("-One only risks it, because\n"
                   "one's survival depends on it.")),
                 (r"$", r"-", 2,
                  ("One only risks it, because-\n"
                   "one's survival depends on it.-")),
                 (r"\Z", r"-", 1,
                  ("One only risks it, because\n"
                   "one's survival depends on it.-")),
                 (r"\W", r"-", 12,
                  ("One-only-risks-it--because-"
                   "one-s-survival-depends-on-it-")),
                 (r".", r"-" , 56, "-" *  56),
                 (r".", r"..", 56, "." * 112),)

        self.assert_replace_all_cases(cases, True)

    def test_replace_all__string(self):
        cases = (("i", "-", 4,
                  ("One only r-sks -t, because\n"
                   "one's surv-val depends on -t.")),
                 (" ", "-", 8,
                  ("One-only-risks-it,-because\n"
                   "one's-survival-depends-on-it.")),
                 ("o", "oo", 3,
                  ("One oonly risks it, because\n"
                   "oone's survival depends oon it.")),
                 ("e", "" , 6,
                  ("On only risks it, bcaus\n"
                   "on's survival dpnds on it.")),
                 ("n", "n", 5,
                  ("One only risks it, because\n"
                   "one's survival depends on it.")))

        self.assert_replace_all_cases(cases, False)

    def test_set_regex(self):
        # pylint: disable=E1103
        flags = re.DOTALL | re.MULTILINE | re.UNICODE
        self.finder.set_regex("test")
        assert self.finder.pattern.pattern == "test"
        assert self.finder.pattern.flags == flags

    def test_set_regex__ignore_case(self):
        # pylint: disable=E1103
        flags = re.DOTALL | re.MULTILINE | re.UNICODE
        self.finder.set_regex("test", re.IGNORECASE)
        assert self.finder.pattern.pattern == "test"
        assert self.finder.pattern.flags == flags | re.IGNORECASE

    def test_set_text__next(self):
        self.finder.set_text("test")
        assert self.finder.text == "test"
        assert self.finder.match_span is None
        assert self.finder.pos == 0

    def test_set_text__previous(self):
        self.finder.set_text("test", False)
        assert self.finder.text == "test"
        assert self.finder.match_span is None
        assert self.finder.pos == 4



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk
import os


class TestOpenAgent(gaupol.TestCase):

    def run__show_encoding_error_dialog(self):
        self.delegate._show_encoding_error_dialog("test")

    def run__show_format_error_dialog(self):
        self.delegate._show_format_error_dialog("test")

    def run__show_io_error_dialog(self):
        self.delegate._show_io_error_dialog("test", "test")

    def run__show_parse_error_dialog(self):
        self.delegate._show_parse_error_dialog("test", aeidon.formats.SUBRIP)

    @aeidon.deco.silent(gaupol.Default)
    def run__show_size_warning_dialog(self):
        self.delegate._show_size_warning_dialog("test", 2)

    @aeidon.deco.silent(gaupol.Default)
    def run__show_sort_warning_dialog(self):
        self.delegate._show_sort_warning_dialog("test", 3)

    @aeidon.deco.silent(gaupol.Default)
    def run__show_translation_warning_dialog(self):
        page = self.application.get_current_page()
        self.delegate._show_translation_warning_dialog(page)

    def setup_method(self, method):
        self.application = self.new_application()
        self.delegate = self.application.open_main.im_self

    @aeidon.deco.monkey_patch(gaupol.FileDialog, "get_filenames")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_append_file_activate(self):
        get_filenames = lambda *args: (self.new_subrip_file(),)
        gaupol.FileDialog.get_filenames = get_filenames
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.application.get_action("append_file").activate()

    def test__on_new_project_activate(self):
        self.application.get_action("new_project").activate()

    @aeidon.deco.monkey_patch(gaupol.FileDialog, "get_filenames")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_open_main_files_activate(self):
        get_filenames = lambda *args: (self.new_subrip_file(),)
        gaupol.FileDialog.get_filenames = get_filenames
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.application.get_action("open_main_files").activate()

    @aeidon.deco.monkey_patch(gaupol.FileDialog, "get_filenames")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_open_translation_file_activate(self):
        get_filenames = lambda *args: (self.new_subrip_file(),)
        gaupol.FileDialog.get_filenames = get_filenames
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        self.application.get_action("open_translation_file").activate()

    @aeidon.deco.monkey_patch(gaupol.FileDialog, "get_filenames")
    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_open_translation_file_activate__changed(self):
        get_filenames = lambda *args: (self.new_subrip_file(),)
        gaupol.FileDialog.get_filenames = get_filenames
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_YES
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        self.application.get_action("open_translation_file").activate()
        page.project.set_text(0, aeidon.documents.TRAN, "")
        self.application.get_action("open_translation_file").activate()

    @aeidon.deco.monkey_patch(gaupol.FileDialog, "get_filenames")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_select_video_file_activate(self):
        get_filenames = lambda *args: (self.new_subrip_file(),)
        gaupol.FileDialog.get_filenames = get_filenames
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        page.project.video_path = self.new_subrip_file()
        self.application.get_action("select_video_file").activate()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__on_split_project_activate(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        page.view.select_rows((3,))
        self.application.get_action("split_project").activate()

    @aeidon.deco.monkey_patch(gaupol.FileDialog, "get_filenames")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_video_button_clicked(self):
        get_filenames = lambda *args: (self.new_subrip_file(),)
        gaupol.FileDialog.get_filenames = get_filenames
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.application.video_button.emit("clicked")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_encoding_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_encoding_error_dialog("test")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_format_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_format_error_dialog("test")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_io_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_io_error_dialog("test", "test")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_parse_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_parse_error_dialog("test", aeidon.formats.SUBRIP)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    @aeidon.deco.silent(gaupol.Default)
    def test__show_size_warning_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_size_warning_dialog("test", 2)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    @aeidon.deco.silent(gaupol.Default)
    def test__show_sort_warning_dialog__no(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_NO
        self.delegate._show_sort_warning_dialog("test", 3)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    @aeidon.deco.silent(gaupol.Default)
    def test__show_sort_warning_dialog__yes(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_YES
        self.delegate._show_sort_warning_dialog("test", 3)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    @aeidon.deco.silent(gaupol.Default)
    def test__show_translation_warning_dialog__no(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_NO
        page = self.application.get_current_page()
        self.delegate._show_translation_warning_dialog(page)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    @aeidon.deco.silent(gaupol.Default)
    def test__show_translation_warning_dialog__yes(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_YES
        page = self.application.get_current_page()
        self.delegate._show_translation_warning_dialog(page)

    def test_add_page(self):
        self.application.add_page(self.new_page())

    def test_add_to_recent_files__main(self):
        self.application.add_to_recent_files(self.new_subrip_file(),
                                             aeidon.formats.SUBRIP,
                                             aeidon.documents.MAIN)

    def test_add_to_recent_files__translation(self):
        self.application.add_to_recent_files(self.new_subrip_file(),
                                             aeidon.formats.SUBRIP,
                                             aeidon.documents.TRAN)

    def test_append_file(self):
        self.application.append_file(self.new_subrip_file())
        self.application.append_file(self.new_subrip_file(), "ascii")

    def test_connect_view_signals(self):
        page = self.application.get_current_page()
        self.application.connect_view_signals(page.view)

    def test_open_main(self):
        path = self.new_subrip_file()
        self.application.open_main(path)

    def test_open_main__again(self):
        path = self.new_subrip_file()
        self.application.open_main(path)
        self.application.open_main(path)

    def test_open_main__ascii(self):
        path = self.new_subrip_file()
        self.application.open_main(path, "ascii")

    def test_open_main__auto(self):
        path = self.new_subrip_file()
        self.application.open_main(path, "auto")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_open_main__format_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        path = self.new_subrip_file()
        open(path, "w").write("xxx\n")
        self.application.open_main(path)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_open_main__io_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        path = self.new_subrip_file()
        os.chmod(path, 0000)
        self.application.open_main(path)
        os.chmod(path, 0777)

    def test_open_main__multiple(self):
        self.application.open_main((self.new_subrip_file(),
                                    self.new_subrip_file(),
                                    self.new_microdvd_file(),
                                    self.new_microdvd_file()))

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_open_main__parse_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_YES
        path = self.new_microdvd_file()
        fobj = open(path, "w")
        fobj.write("{10}{20}Testing...\n")
        fobj.write("{20}{30}Testing...\n")
        fobj.write("{30}{40}Testing...\n")
        fobj.write("{xx}{yy}Testing...\n")
        fobj.close()
        self.application.open_main(path)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_open_main__sort_warning(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_YES
        path = self.new_microdvd_file()
        fobj = open(path, "w")
        fobj.write("{30}{40}Testing...\n")
        fobj.write("{40}{50}Testing...\n")
        fobj.write("{10}{20}Testing...\n")
        fobj.write("{20}{30}Testing...\n")
        fobj.close()
        self.application.open_main(path)

    def test_open_translation(self):
        path = self.new_subrip_file()
        self.application.open_translation(path)

    def test_open_translation__align_method(self):
        path = self.new_subrip_file()
        align_method = aeidon.align_methods.POSITION
        self.application.open_translation(path, align_method=align_method)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Text searching actions for :class:`gaupol.Application`."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._


class FindAndReplaceAction(gaupol.Action):

    """Search for and replace text."""

    def __init__(self):
        """Initialize a :class:`FindAndReplaceAction` object."""
        gaupol.Action.__init__(self, "find_and_replace")
        self.props.label = _("_Find And Replace\342\200\246")
        self.props.short_label = _("Find")
        self.props.stock_id = gtk.STOCK_FIND_AND_REPLACE
        self.props.tooltip = _("Search for and replace text")
        self.accelerator = "<Control>F"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class FindNextAction(gaupol.Action):

    """Search forwards for same text."""

    def __init__(self):
        """Initialize a :class:`FindNextAction` object."""
        gaupol.Action.__init__(self, "find_next")
        self.props.label = _("Find _Next")
        self.props.tooltip = _("Search forwards for same text")
        self.accelerator = "<Control>G"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(application.pattern)


class FindPreviousAction(gaupol.Action):

    """Search backwards for same text."""

    def __init__(self):
        """Initialize a :class:`FindPreviousAction` object."""
        gaupol.Action.__init__(self, "find_previous")
        self.props.label = _("Find Pre_vious")
        self.props.tooltip = _("Search backwards for same text")
        self.accelerator = "<Shift><Control>G"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(application.pattern)


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""File and project actions for :class:`gaupol.Application`."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._


class AppendFileAction(gaupol.Action):

    """Append subtitles from file to the current project."""

    def __init__(self):
        """Initialize an :class:`AppendFileAction` object."""
        gaupol.Action.__init__(self, "append_file")
        self.props.label = _("_Append File\342\200\246")
        self.props.stock_id = gtk.STOCK_ADD
        self.props.tooltip = _("Append subtitles from file "
                               "to the current project")

        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class CloseAllProjectsAction(gaupol.Action):

    """Close all open projects."""

    def __init__(self):
        """Initialize a :class:`CloseAllProjectsAction` object."""
        gaupol.Action.__init__(self, "close_all_projects")
        self.props.label = _("_Close All")
        self.props.stock_id = gtk.STOCK_CLOSE
        self.props.tooltip =  _("Close all open projects")
        self.accelerator = "<Shift><Control>W"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(application.pages)


class CloseProjectAction(gaupol.Action):

    """Close project."""

    def __init__(self):
        """Initialize a :class:`CloseProjectAction` object."""
        gaupol.Action.__init__(self, "close_project")
        self.props.label = _("_Close")
        self.props.stock_id = gtk.STOCK_CLOSE
        self.props.tooltip = _("Close the current project")
        self.accelerator = "<Control>W"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class EditHeadersAction(gaupol.Action):

    """Edit file headers."""

    def __init__(self):
        """Initialize an :class:`EditHeadersAction` object."""
        gaupol.Action.__init__(self, "edit_headers")
        self.props.label = _("_Headers")
        self.props.stock_id = gtk.STOCK_PROPERTIES
        self.props.tooltip = _("Edit file headers")
        self.accelerator = "<Alt>Return"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        has_header_found = False
        if page.project.main_file is not None:
            if page.project.main_file.format.has_header:
                has_header_found = True
        if page.project.tran_file is not None:
            if page.project.tran_file.format.has_header:
                has_header_found = True
        aeidon.util.affirm(has_header_found)


class NewProjectAction(gaupol.Action):

    """Create a new project."""

    def __init__(self):
        """Initialize a :class:`NewProjectAction` object."""
        gaupol.Action.__init__(self, "new_project")
        self.props.label = _("_New")
        self.props.stock_id = gtk.STOCK_NEW
        self.props.tooltip = _("Create a new project")
        self.accelerator = "<Control>N"
        self.action_group = "main-safe"


class OpenMainFilesAction(gaupol.Action):

    """Open main files."""

    __gtype_name__ = "OpenMainFilesAction"

    def __init__(self):
        """Initialize an :class:`OpenMainFilesAction` object."""
        gaupol.Action.__init__(self, "open_main_files")
        self.props.is_important = True
        self.props.label = _("_Open\342\200\246")
        self.props.short_label = _("Open")
        self.props.stock_id = gtk.STOCK_OPEN
        self.props.tooltip = _("Open main files")
        self.accelerator = "<Control>O"
        self.action_group = "main-safe"
        self.set_tool_item_type(gtk.MenuToolButton)


class OpenMainFilesRecentAction(gaupol.RecentAction):

    """Open main files."""

    group = "gaupol-main"

    def __init__(self):
        """Initialize an :class:`OpenMainFilesRecentAction` object."""
        gaupol.RecentAction.__init__(self, "open_main_files_recent")
        self.props.is_important = True
        self.props.label = _("_Open\342\200\246")
        self.props.short_label = _("Open")
        self.props.stock_id = gtk.STOCK_OPEN
        self.props.tooltip = _("Open main files")
        self.action_group = "main-safe"
        self.set_tool_item_type(gtk.MenuToolButton)


class OpenRecentMainFileAction(gaupol.RecentAction):

    """Show the recent main file menu."""

    group = "gaupol-main"

    def __init__(self):
        """Initialize a :class:`OpenRecentMainFileAction` object."""
        gaupol.RecentAction.__init__(self, "open_recent_main_file")
        self.props.is_important = True
        self.props.label = _("Open _Recent")
        self.action_group = "main-safe"


class OpenRecentTranslationFileAction(gaupol.RecentAction):

    """Show the recent translation file menu."""

    group = "gaupol-translation"

    def __init__(self):
        """Initialize a :class:`OpenRecentTranslationFileAction` object."""
        gaupol.RecentAction.__init__(self, "open_recent_translation_file")
        self.props.is_important = False
        self.props.label = _("Open R_ecent Translation")
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class OpenTranslationFileAction(gaupol.Action):

    """Open a translation file."""

    def __init__(self):
        """Initialize an :class:`OpenTranslationFileAction` object."""
        gaupol.Action.__init__(self, "open_translation_file")
        self.props.label = _("Open _Translation\342\200\246")
        self.props.short_label = _("Open Translation")
        self.props.stock_id = gtk.STOCK_OPEN
        self.props.tooltip = _("Open a translation file")
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class QuitAction(gaupol.Action):

    """Quit Gaupol."""

    def __init__(self):
        """Initialize a :class:`QuitAction` object."""
        gaupol.Action.__init__(self, "quit")
        self.props.label = _("_Quit")
        self.props.stock_id = gtk.STOCK_QUIT
        self.props.tooltip = _("Quit Gaupol")
        self.accelerator = "<Control>Q"
        self.action_group = "main-safe"


class SaveAllDocumentsAction(gaupol.Action):

    """Save all open documents."""

    def __init__(self):
        """Initialize a :class:`SaveAllDocumentsAction` object."""
        gaupol.Action.__init__(self, "save_all_documents")
        self.props.label = _("_Save All")
        self.props.stock_id = gtk.STOCK_SAVE
        self.props.tooltip = _("Save all open documents")
        self.accelerator = "<Shift><Control>L"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(application.pages)


class SaveAllDocumentsAsAction(gaupol.Action):

    """Save all open documents with different properties."""

    def __init__(self):
        """Initialize a :class:`SaveAllDocumentsAsAction` object."""
        gaupol.Action.__init__(self, "save_all_documents_as")
        self.props.label = _("Save _All As\342\200\246")
        self.props.stock_id = gtk.STOCK_SAVE_AS
        self.props.tooltip = _("Save all open documents with "
                               "different properties")

        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm([x for x in application.pages
                            if x.project.main_file is not None])


class SaveMainDocumentAction(gaupol.Action):

    """Save the current main document."""

    def __init__(self):
        """Initialize a :class:`SaveMainDocumentAction` object."""
        gaupol.Action.__init__(self, "save_main_document")
        self.props.is_important = True
        self.props.label = _("_Save")
        self.props.stock_id = gtk.STOCK_SAVE
        self.props.tooltip = _("Save the current main document")
        self.accelerator = "<Control>S"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class SaveMainDocumentAsAction(gaupol.Action):

    """Save the current main document with a different name."""

    def __init__(self):
        """Initialize a :class:`SaveMainDocumentAsAction` object."""
        gaupol.Action.__init__(self, "save_main_document_as")
        self.props.label = _("Save _As\342\200\246")
        self.props.short_label = _("Save As")
        self.props.stock_id = gtk.STOCK_SAVE_AS
        self.props.tooltip = _("Save the current main document "
                               "with a different name")

        self.accelerator = "<Shift><Control>S"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class SaveTranslationDocumentAction(gaupol.Action):

    """Save the current translation document."""

    def __init__(self):
        """Initialize a :class:`SaveTranslationDocumentAction` object."""
        gaupol.Action.__init__(self, "save_translation_document")
        self.props.label = _("Save Trans_lation")
        self.props.stock_id = gtk.STOCK_SAVE
        self.props.tooltip = _("Save the current translation document")
        self.accelerator = "<Control>T"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class SaveTranslationDocumentAsAction(gaupol.Action):

    """Save the current translation document with a different name."""

    def __init__(self):
        """Initialize a :class:`SaveTranslationDocumentAsAction` object."""
        gaupol.Action.__init__(self, "save_translation_document_as")
        self.props.label = _("Save Translat_ion As\342\200\246")
        self.props.short_label = _("Save Translation As")
        self.props.stock_id = gtk.STOCK_SAVE_AS
        self.props.tooltip = _("Save the current translation document "
                               "with a different name")

        self.accelerator = "<Shift><Control>T"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class SelectVideoFileAction(gaupol.Action):

    """Select a video file."""

    def __init__(self):
        """Initialize a :class:`SelectVideoFileAction` object."""
        gaupol.Action.__init__(self, "select_video_file")
        self.props.label = _("Select _Video\342\200\246")
        self.props.short_label = _("Video")
        self.props.stock_id = gtk.STOCK_FILE
        self.props.tooltip = _("Select a video file")
        self.action_group = "main-safe"
        self.widgets = ("video_button",)

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class SplitProjectAction(gaupol.Action):

    """Split the current project in two."""

    def __init__(self):
        """Initialize a :class:`SplitProjectAction` object."""
        gaupol.Action.__init__(self, "split_project")
        self.props.label = _("Spli_t Project\342\200\246")
        self.props.tooltip = _("Split the current project in two")
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(len(page.project.subtitles) > 1)


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk
import os


class TestSaveAgent(gaupol.TestCase):

    def run__show_encoding_error_dialog(self):
        self.delegate._show_encoding_error_dialog("test", "ascii")

    def run__show_io_error_dialog(self):
        self.delegate._show_io_error_dialog("test", "test")

    def setup_method(self, method):
        self.application = self.new_application()
        self.delegate = self.application.save_main.im_self

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_encoding_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_encoding_error_dialog("test", "ascii")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_io_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_io_error_dialog("test", "test")

    def test__on_save_all_documents_activate(self):
        self.application.get_action("save_all_documents").activate()

    def test__on_save_main_document_activate(self):
        self.application.get_action("save_main_document").activate()

    @aeidon.deco.monkey_patch(gaupol.SaveDialog, "get_filename")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_save_main_document_as_activate(self):
        get_filename = lambda *args: self.new_subrip_file()
        gaupol.SaveDialog.get_filename = get_filename
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.application.get_action("save_main_document_as").activate()

    def test__on_save_translation_document_activate(self):
        self.application.get_action("save_translation_document").activate()

    @aeidon.deco.monkey_patch(gaupol.SaveDialog, "get_filename")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_save_translation_document_as_activate(self):
        get_filename = lambda *args: self.new_subrip_file()
        gaupol.SaveDialog.get_filename = get_filename
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.application.get_action("save_translation_document_as").activate()

    def test_save_main(self):
        page = self.application.get_current_page()
        self.application.save_main(page)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_save_main__io_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        os.chmod(page.project.main_file.path, 0000)
        self.raises(gaupol.Default, self.application.save_main, page)
        os.chmod(page.project.main_file.path, 0777)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_save_main__unicode_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        page.project.main_file.encoding = "ascii"
        page.project.set_text(0, aeidon.documents.MAIN, "\303\266")
        self.raises(gaupol.Default, self.application.save_main, page)

    @aeidon.deco.monkey_patch(gaupol.SaveDialog, "get_filename")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test_save_main__untitled(self):
        get_filename = lambda *args: self.new_subrip_file()
        gaupol.SaveDialog.get_filename = get_filename
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.application.get_action("new_project").activate()
        page = self.application.get_current_page()
        self.application.save_main(page)

    @aeidon.deco.monkey_patch(gaupol.SaveDialog, "get_filename")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test_save_main_as(self):
        get_filename = lambda *args: self.new_subrip_file()
        gaupol.SaveDialog.get_filename = get_filename
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        self.application.save_main_as(page)

    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test_save_main_as__cancel(self):
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_CANCEL
        page = self.application.get_current_page()
        self.raises(gaupol.Default, self.application.save_main_as, page)

    def test_save_translation(self):
        page = self.application.get_current_page()
        self.application.save_translation(page)

    @aeidon.deco.monkey_patch(gaupol.SaveDialog, "get_filename")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test_save_translation__untitled(self):
        get_filename = lambda *args: self.new_subrip_file()
        gaupol.SaveDialog.get_filename = get_filename
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        self.application.get_action("new_project").activate()
        page = self.application.get_current_page()
        self.application.save_translation(page)

    @aeidon.deco.monkey_patch(gaupol.SaveDialog, "get_filename")
    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test_save_translation_as(self):
        get_filename = lambda *args: self.new_subrip_file()
        gaupol.SaveDialog.get_filename = get_filename
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        page = self.application.get_current_page()
        self.application.save_translation_as(page)



# Copyright (C) 2006-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for splitting a project in two."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("SplitDialog",)


class SplitDialog(gaupol.BuilderDialog):

    """Dialog for splitting a project in two."""

    __metaclass__ = aeidon.Contractual
    _widgets = ("subtitle_spin",)

    def __init___require(self, parent, application):
        page = application.get_current_page()
        assert page is not None
        assert len(page.project.subtitles) > 1

    def __init__(self, parent, application):
        """Initialize a :class:`SplitDialog` object."""
        gaupol.BuilderDialog.__init__(self, "split-dialog.ui")
        self.application = application
        self._init_subtitle_spin()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_OK)

    def _init_subtitle_spin(self):
        """Initialize the subtitle spin button."""
        page = self.application.get_current_page()
        rows = page.view.get_selected_rows()
        subtitle = (rows[0] + 1 if rows else 2)
        self._subtitle_spin.set_value(subtitle)
        self._subtitle_spin.set_range(2, len(page.project.subtitles))

    def _on_subtitle_spin_value_changed(self, spin_button):
        """Select matching row in the view."""
        page = self.application.get_current_page()
        row = self._subtitle_spin.get_value_as_int() - 1
        page.view.set_focus(row, None)

    def _on_response(self, dialog, response):
        """Split the current project if OK'd."""
        if response == gtk.RESPONSE_OK:
            self._split_project()

    def _remove_from_source(self, page, index):
        """Remove rows starting at `index` from `page`."""
        indices = range(index, len(page.project.subtitles))
        page.project.block("action-done")
        page.project.remove_subtitles(indices)
        page.project.set_action_description(
            aeidon.registers.DO, _("Splitting project"))
        page.project.unblock("action-done")

    def _shift_destination(self, src, dst):
        """Shift subtitles in `dst` page."""
        amount = src.project.subtitles[-1].end
        if isinstance(amount, basestring):
            amount = (amount[1:] if amount.startswith("-") else "-%s" % amount)
        if isinstance(amount, (int, float)):
            amount = -1 * amount
        dst.project.shift_positions(None, amount, register=None)

    def _split_project(self):
        """Split the current project in two."""
        gaupol.util.set_cursor_busy(self.application.window)
        index = self._subtitle_spin.get_value_as_int() - 1
        src = self.application.get_current_page()
        dst = gaupol.Page(self.application.counter.next())
        subtitles = [x.copy() for x in src.project.subtitles[index:]]
        indices = range(len(subtitles))
        dst.project.insert_subtitles(indices, subtitles)
        dst.reload_view_all()
        self._remove_from_source(src, index)
        self._shift_destination(src, dst)
        self.application.add_page(dst)
        amount = len(dst.project.subtitles)
        name = dst.untitle
        message = _('Split %(amount)d subtitles to project "%(name)s"')
        self.application.flash_message(message % locals())
        gaupol.util.set_cursor_normal(self.application.window)



# Copyright (C) 2005-2007,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Processing texts."""

import aeidon
import gaupol


class TextAgent(aeidon.Delegate):

    """Processing texts."""

    @aeidon.deco.export
    def _on_check_spelling_activate(self, *args):
        """Check for incorrect spelling."""
        gaupol.util.set_cursor_busy(self.window)
        try: # Fails if no dictionary for conf.spell_check.language.
            dialog = gaupol.SpellCheckDialog(self.window, self)
        except ValueError:
            return gaupol.util.set_cursor_normal(self.window)
        gaupol.util.set_cursor_normal(self.window)
        gaupol.util.flash_dialog(dialog)

    @aeidon.deco.export
    def _on_configure_spell_check_activate(self, *args):
        """Set languages and spell-check targets."""
        gaupol.util.set_cursor_busy(self.window)
        dialog = gaupol.LanguageDialog(self.window)
        gaupol.util.set_cursor_normal(self.window)
        gaupol.util.flash_dialog(dialog)
        self.update_gui()

    @aeidon.deco.export
    def _on_correct_texts_activate(self, *args):
        """Find and correct errors in texts."""
        gaupol.util.set_cursor_busy(self.window)
        assistant = gaupol.TextAssistant(self.window, self)
        gaupol.util.set_cursor_normal(self.window)
        assistant.show()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""MPL2 file."""

import aeidon
import re

__all__ = ("MPL2",)


class MPL2(aeidon.SubtitleFile):

    """MPL2 file."""

    _re_line = re.compile(r"^\[(-?\d+)\]\[(-?\d+)\](.*?)$")
    format = aeidon.formats.MPL2
    mode = aeidon.modes.TIME

    def read(self):
        """Read file and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        """
        subtitles = []
        for line in self._read_lines():
            match = self._re_line.match(line)
            if match is None: continue
            subtitle = self._get_subtitle()
            subtitle.start = float(match.group(1)) / 10
            subtitle.end = float(match.group(2)) / 10
            subtitle.main_text = match.group(3).replace("|", "\n")
            subtitles.append(subtitle)
        return subtitles

    def write_to_file(self, subtitles, doc, fobj):
        """Write `subtitles` from `doc` to `fobj`.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        for subtitle in subtitles:
            fobj.write("[%.0f]" % (subtitle.start_seconds * 10))
            fobj.write("[%.0f]" % (subtitle.end_seconds * 10))
            fobj.write(subtitle.get_text(doc).replace("\n", "|"))
            fobj.write(self.newline.value)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol


class TestMenuAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        self.delegate = self.application.set_menu_notify_events.im_self

    def test_on_redo_button_show_menu(self):
        page = self.application.get_current_page()
        page.project.remove_subtitles((0,))
        page.project.remove_subtitles((0,))
        page.project.undo(2)
        item = self.application.get_tool_item("redo_action")
        item.emit("show-menu")
        item.get_menu().get_children()[0].activate()

    def test_on_show_projects_menu_activate(self):
        self.application.get_action("show_projects_menu").activate()
        self.application.get_action("show_projects_menu").activate()
        self.application.get_action("show_projects_menu").activate()

    def test_on_undo_button_show_menu(self):
        page = self.application.get_current_page()
        page.project.remove_subtitles((0,))
        page.project.remove_subtitles((0,))
        item = self.application.get_tool_item("undo_action")
        item.emit("show-menu")
        item.get_menu().get_children()[0].activate()

    def test_set_menu_notify_events(self):
        self.application.set_menu_notify_events("main-safe")
        self.application.set_menu_notify_events("main-unsafe")
        self.application.set_menu_notify_events("projects")



# Copyright (C) 2005-2007,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for editing text of a single subtitle."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._


class TextEditDialog(gtk.Dialog):

    """Dialog for editing text of a single subtitle."""

    def __init__(self, parent, text=""):
        """Initialize a :class:`TextEditDialog` object."""
        gtk.Dialog.__init__(self)
        self._text_view = None
        self._init_dialog(parent)
        self._init_text_view()
        self.set_text(text)

    def _init_dialog(self, parent):
        """Initialize the dialog."""
        self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.set_default_response(gtk.RESPONSE_OK)
        self.set_has_separator(False)
        self.set_transient_for(parent)
        self.set_border_width(6)
        self.set_modal(True)
        self.set_title(_("Edit Text"))

    def _init_text_view(self):
        """Initialize the text view."""
        self._text_view = gtk.TextView()
        gaupol.util.prepare_text_view(self._text_view)
        self._text_view.set_wrap_mode(gtk.WRAP_NONE)
        self._text_view.set_accepts_tab(False)
        self._text_view.set_left_margin(6)
        self._text_view.set_right_margin(6)
        font = gaupol.util.get_font()
        gaupol.util.scale_to_size(self._text_view, 70, 6, font)
        scroller = gtk.ScrolledWindow()
        scroller.set_border_width(6)
        scroller.set_policy(*((gtk.POLICY_AUTOMATIC,) * 2))
        scroller.set_shadow_type(gtk.SHADOW_IN)
        scroller.add(self._text_view)
        vbox = self.get_child()
        vbox.add(scroller)
        vbox.show_all()

    def get_text(self):
        """Return text in the text view."""
        text_buffer = self._text_view.get_buffer()
        bounds = text_buffer.get_bounds()
        return text_buffer.get_text(*bounds)

    def set_text(self, text):
        """Set `text` to the text view."""
        text_buffer = self._text_view.get_buffer()
        text_buffer.set_text(text)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Text markup for the SubRip format."""

import aeidon
import re

__all__ = ("SubRip",)


class SubRip(aeidon.Markup):

    """Text markup for the SubRip format.

    SubRip format is assumed (based on the SubRip application GUI) to contain
    the following HTML-style tags, in either lower- or upper case.

     * ``<b>.........................</b>``
     * ``<i>.........................</i>``
     * ``<u>.........................</u>``
     * ``<font color="#RRGGBB">...</font>``
    """

    _flags = re.DOTALL | re.MULTILINE | re.UNICODE | re.IGNORECASE
    format = aeidon.formats.SUBRIP

    def _main_decode(self, text):
        """Return `text` with decodable markup decoded."""
        text = self._decode_b(text, r"<b>(.*?)</b>", 1)
        text = self._decode_i(text, r"<i>(.*?)</i>", 1)
        text = self._decode_u(text, r"<u>(.*?)</u>", 1)
        pattern = r'<font color="#([0-9a-fA-F]{6})">(.*?)</font>'
        return self._decode_c(text, pattern, 1, 2)

    def bolden(self, text, bounds=None):
        """Return bolded `text`."""
        a, z = bounds or (0, len(text))
        return "".join((text[:a], "<b>%s</b>" % text[a:z], text[z:]))

    def clean(self, text):
        """Return `text` with less ugly markup."""
        # Remove tags that are immeadiately closed after opening.
        text = self._substitute(text, r"<([a-z]+)[^<]*?>( *)</\1>", r"\2")
        # Remove tags that are immeadiately opened after closing.
        text = self._substitute(text, r"</([a-z]+)>( *)<\1[^<]*?>", r"\2")
        # Remove or relocate space right after an opening tag.
        text = self._substitute(text, r" ?(<(?!/)[^>]+?>) ", r" \1")
        # Remove or relocate space right before a closing tag.
        return self._substitute(text, r" (</[^>]+?>) ?", r"\1 ")

    def colorize(self, text, color, bounds=None):
        """Return `text` colorized to hexadecimal value."""
        a, z = bounds or (0, len(text))
        target = '<font color="#%s">%s</font>' % (color, text[a:z])
        return "".join((text[:a], target, text[z:]))

    @property
    def italic_tag(self):
        """Regular expression for an italic markup tag."""
        return self._get_regex(r"</?i>")

    def italicize(self, text, bounds=None):
        """Return italicized `text`."""
        a, z = bounds or (0, len(text))
        return "".join((text[:a], "<i>%s</i>" % text[a:z], text[z:]))

    @property
    def tag(self):
        """Regular expression for any markup tag."""
        return self._get_regex(r"<.*?>")

    def underline(self, text, bounds=None):
        """Return underlined `text`."""
        a, z = bounds or (0, len(text))
        return "".join((text[:a], "<u>%s</u>" % text[a:z], text[z:]))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for displaying credits and information."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("AboutDialog",)

_license = (
    "Gaupol is free software: you can redistribute it and/or modify it under "
    "the terms of the GNU General Public License as published by the Free "
    "Software Foundation, either version 3 of the License, or (at your "
    "option) any later version.\n\n"
    "Gaupol is distributed in the hope that it will be useful, but WITHOUT "
    "ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or "
    "FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for "
    "more details.\n\n"
    "You should have received a copy of the GNU General Public License along "
    "with Gaupol. If not, see <http://www.gnu.org/licenses/>.")


class AboutDialog(gtk.AboutDialog):

    """Dialog for displaying credits and information."""

    def __init__(self, parent):
        """Initialize an :class:`AboutDialog` object."""
        gtk.AboutDialog.__init__(self)
        gtk.about_dialog_set_url_hook(self._on_url_clicked)
        self.set_transient_for(parent)
        self.set_title(_("About Gaupol"))
        self.set_program_name("Gaupol")
        self.set_version(gaupol.__version__)
        self.set_copyright(u"Copyright \xa9 2005-2011 Osmo Salomaa")
        self.set_comments(_("Subtitle editor"))
        self.set_license(_license)
        self.set_wrap_license(True)
        self.set_website(gaupol.HOMEPAGE_URL)
        self.set_website_label(_("Gaupol Website"))
        self.set_authors(("Osmo Salomaa <otsaloma@iki.fi>",))
        self.set_artists(("Osmo Salomaa <otsaloma@iki.fi>",))
        self.set_logo_icon_name("gaupol")

        # Translators: This is a special message that shouldn't be translated
        # literally. It is used in the about dialog to give credits to the
        # translators. Thus, you should translate it to your name and email
        # address. You can also include other translators who have contributed
        # to this translation; in that case, please write them on separate
        # lines seperated by newlines (\n).
        self.set_translator_credits(_("translator-credits"))

    def _on_url_clicked(self, dialog, url):
        """Open website in a web browser."""
        gaupol.util.show_uri(url)



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Setting values of single subtitle fields."""

import aeidon
import bisect
_ = aeidon.i18n._


class SetAgent(aeidon.Delegate):

    """Setting values of single subtitle fields."""

    __metaclass__ = aeidon.Contractual

    def _move_if_needed_require(self, index):
        assert 0 <= index < len(self.subtitles)

    def _move_if_needed(self, index):
        """Move subtitle for correct order and return new index."""
        subtitle = self.subtitles[index]
        subtitles = self.subtitles[:index] + self.subtitles[index + 1:]
        new_index = bisect.bisect_right(subtitles, subtitle)
        if new_index == index: return new_index
        subtitle = self.subtitles.pop(index)
        self.emit("subtitles-removed", (index,))
        self.subtitles.insert(new_index, subtitle)
        self.emit("subtitles-inserted", (new_index,))
        return new_index

    def set_duration_require(self, index, value, register=-1):
        assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def set_duration(self, index, value, register=-1):
        """Set the value of duration position.

        `value` can be a string to set duration as time, an integer to set as
        frame or a float to set as seconds.
        """
        subtitle = self.subtitles[index]
        orig_end = subtitle.end
        subtitle.duration = value
        if subtitle.end == orig_end: return
        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Editing position")
        action.revert_function = self.set_end
        action.revert_args = (index, orig_end)
        self.register_action(action)
        self.emit("positions-changed", (index,))

    def set_end_require(self, index, value, register=-1):
        assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def set_end(self, index, value, register=-1):
        """Set the value of end position.

        `value` can be a string to set duration as time, an integer to set as
        frame or a float to set as seconds.
        """
        subtitle = self.subtitles[index]
        orig_value = subtitle.end
        subtitle.end = value
        if subtitle.end == orig_value: return
        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Editing position")
        action.revert_function = self.set_end
        action.revert_args = (index, orig_value)
        self.register_action(action)
        self.emit("positions-changed", (index,))

    def set_main_text_require(self, index, value, register=-1):
        assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def set_main_text(self, index, value, register=-1):
        """Set the value of main document's text."""
        return self.set_text(index,
                             aeidon.documents.MAIN,
                             value,
                             register=register)

    def set_start_require(self, index, value, register=-1):
        assert 0 <= index < len(self.subtitles)

    def set_start_ensure(self, return_value, index, value, register=-1):
        for i in range(len(self.subtitles) - 1):
            assert self.subtitles[i] <= self.subtitles[i + 1]

    @aeidon.deco.export
    @aeidon.deco.revertable
    @aeidon.deco.notify_frozen
    def set_start(self, index, value, register=-1):
        """Set the value of start position.

        `value` can be a string to set duration as time, an integer to set as
        frame or a float to set as seconds.
        """
        subtitle = self.subtitles[index]
        orig_value = subtitle.start
        subtitle.start = value
        if subtitle.start == orig_value: return
        index = self._move_if_needed(index)
        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Editing position")
        action.revert_function = self.set_start
        action.revert_args = (index, orig_value)
        self.register_action(action)
        self.emit("positions-changed", (index,))

    def set_text_require(self, index, doc, value, register=-1):
        assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def set_text(self, index, doc, value, register=-1):
        """Set the value of `doc`'s text."""
        value = unicode(value)
        subtitle = self.subtitles[index]
        orig_value = subtitle.get_text(doc)
        if value == orig_value: return
        subtitle.set_text(doc, value)
        action = self.new_revertable_action(register)
        action.docs = (doc,)
        action.description = _("Editing text")
        action.revert_function = self.set_text
        action.revert_args = (index, doc, orig_value)
        self.register_action(action)
        signal = self.get_text_signal(doc)
        self.emit(signal, (index,))

    def set_translation_text_require(self, index, value, register=-1):
        assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def set_translation_text(self, index, value, register=-1):
        """Set the value of translation document's text."""
        return self.set_text(index,
                             aeidon.documents.TRAN,
                             value,
                             register=register)



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Creating and removing temporary files and directories."""

import aeidon
import atexit
import os
import tempfile

_handles = {}


def close(path):
    """Close the OS-level handle to the file at `path`."""
    os.close(_handles[path])
    del _handles[path]

def create_ensure(value, suffix=""):
    assert os.path.isfile(value)

@aeidon.deco.contractual
def create(suffix=""):
    """Create a new temporary file and return its path."""
    handle, path = tempfile.mkstemp(suffix, "gaupol-")
    _handles[path] = handle
    return path

def create_directory_ensure(value, suffix=""):
    assert os.path.isdir(value)

@aeidon.deco.contractual
def create_directory(suffix=""):
    """Create a new temporary directory and return its path."""
    return tempfile.mkdtemp(suffix, "gaupol-")

def get_handle(path):
    """Return the OS-level handle to the file at path."""
    return _handles[path]

@aeidon.deco.silent(OSError)
def remove(path):
    """Remove temporary file at `path` after closing its handle."""
    if path in _handles:
        close(path)
    os.remove(path)

def remove_all():
    """Remove all temporary files after closing their handles."""
    for path in set(_handles.keys()):
        remove(path)

@aeidon.deco.silent(OSError)
def remove_directory(root):
    """Remove temporary directory at `root` and all its contents."""
    for name in os.listdir(root):
        path = os.path.join(root, name)
        if os.path.isdir(path):
            remove_directory(path)
        elif os.path.isfile(path):
            remove(path)
    os.rmdir(root)

atexit.register(remove_all)



# Copyright (C) 2005-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Enumerations for subtitle file format types.

Mime-types of subtitle file formats conform to those defined in
freedesktop.org_'s shared-mime-info_ package. For all formats not included in
``shared-mime-info``, ``text/plain`` is used as the mime-type.

.. _freedesktop.org: http://www.freedesktop.org/
.. _shared-mime-info: http://freedesktop.org/wiki/Software/shared-mime-info
"""

import aeidon

__all__ = ("formats",)


class AdvSubStationAlpha(aeidon.EnumerationItem):

    container = "ssa"
    extension = ".ass"
    has_header = True
    identifier = r"^ScriptType:\s*[vV]4.00\+\s*$"
    label = "Advanced Sub Station Alpha"
    mime_type = "text/x-ssa"


class MicroDVD(aeidon.EnumerationItem):

    container = None
    extension = ".sub"
    has_header = True
    identifier = r"^\{-?\d+\}\{-?\d+\}"
    label = "MicroDVD"
    mime_type = "text/x-microdvd"


class MPL2(aeidon.EnumerationItem):

    container = None
    extension = ".txt"
    has_header = False
    identifier = r"^\[-?\d+\]\[-?\d+\]"
    label = "MPL2"
    mime_type = "text/plain"


class MPsub(aeidon.EnumerationItem):

    container = None
    extension = ".sub"
    has_header = True
    identifier = r"^FORMAT=(TIME|[\d.]+)\s*$"
    label = "MPsub"
    mime_type = "text/x-mpsub"


class SubRip(aeidon.EnumerationItem):

    container = "subrip"
    extension = ".srt"
    has_header = False
    identifier = (r"^-?\d\d:\d\d:\d\d,\d\d\d -->"
                  r" -?\d\d:\d\d:\d\d,\d\d\d"
                  r"(  X1:\d+ X2:\d+ Y1:\d+ Y2:\d+)?\s*$")

    label = "SubRip"
    mime_type = "application/x-subrip"


class SubStationAlpha(aeidon.EnumerationItem):

    container = "ssa"
    extension = ".ssa"
    has_header = True
    identifier = r"^ScriptType:\s*[vV]4.00\s*$"
    label = "Sub Station Alpha"
    mime_type = "text/x-ssa"


class SubViewer2(aeidon.EnumerationItem):

    container = None
    extension = ".sub"
    has_header = True
    identifier = (r"^-?\d\d:\d\d:\d\d\.\d\d"
                  r",-?\d\d:\d\d:\d\d\.\d\d\s*$")

    label = "SubViewer 2.0"
    mime_type = "text/x-subviewer"


class TMPlayer(aeidon.EnumerationItem):

    container = None
    extension = ".txt"
    has_header = False
    identifier = r"^-?\d?\d:\d\d:\d\d:"
    label = "TMPlayer"
    mime_type = "text/plain"


formats = aeidon.Enumeration()
formats.ASS = AdvSubStationAlpha()
formats.MICRODVD = MicroDVD()
formats.MPL2 = MPL2()
formats.MPSUB = MPsub()
formats.SUBRIP = SubRip()
formats.SSA = SubStationAlpha()
formats.SUBVIEWER2 = SubViewer2()
formats.TMPLAYER = TMPlayer()



# Copyright (C) 2005-2008 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Enumerations for action target types."""

import aeidon

__all__ = ("targets",)

targets = aeidon.Enumeration()
targets.SELECTED = aeidon.EnumerationItem()
targets.CURRENT = aeidon.EnumerationItem()
targets.ALL = aeidon.EnumerationItem()



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk
import time


class TestPreviewAgent(gaupol.TestCase):

    def run__show_encoding_error_dialog(self):
        self.delegate._show_encoding_error_dialog()

    def run__show_io_error_dialog(self):
        self.delegate._show_io_error_dialog("test")

    def run__show_process_error_dialog(self):
        self.delegate._show_process_error_dialog("test")

    def setup_method(self, method):
        gaupol.conf.preview.custom_command = "echo $SUBFILE"
        gaupol.conf.preview.use_custom_command = True
        self.application = self.new_application()
        page = self.application.get_current_page()
        page.project.video_path = self.new_subrip_file()
        self.delegate = self.application.preview.im_self

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_encoding_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_encoding_error_dialog()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_io_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_io_error_dialog("test")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_process_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.delegate._show_process_error_dialog("test")

    def test_on_preview_activate__main(self):
        page = self.application.get_current_page()
        page.view.set_focus(0, page.view.columns.MAIN_TEXT)
        self.application.get_action("preview").activate()

    def test_on_preview_activate__translation(self):
        page = self.application.get_current_page()
        page.view.set_focus(0, page.view.columns.TRAN_TEXT)
        self.application.get_action("preview").activate()

    def test_preview__main(self):
        page = self.application.get_current_page()
        doc = aeidon.documents.MAIN
        self.application.preview(page, 13, doc)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_preview__process_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        gaupol.conf.preview.custom_command = "xxxxx"
        page = self.application.get_current_page()
        doc = aeidon.documents.MAIN
        self.application.preview(page, 13, doc)
        time.sleep(1)
        gaupol.util.iterate_main()

    def test_preview__translation(self):
        page = self.application.get_current_page()
        doc = aeidon.documents.TRAN
        self.application.preview(page, 13, doc)

    def test_preview__unicode_error(self):
        page = self.application.get_current_page()
        doc = aeidon.documents.MAIN
        page.project.set_text(0, doc, "\303\266")
        page.project.main_file.encoding = "ascii"
        self.application.preview(page, 13, doc)

    def test_preview_changes(self):
        page = self.application.get_current_page()
        subtitles = [x.copy() for x in page.project.subtitles]
        framerate = page.project.framerate
        self.application.preview_changes(page,
                                         3,
                                         aeidon.documents.TRAN,
                                         page.project.remove_subtitles,
                                         ((0, 1, 2),))

        assert page.project.subtitles == subtitles
        assert page.project.framerate == framerate



# Copyright (C) 2005-2008 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol


class TestModule(gaupol.TestCase):

    def test_attributes(self):
        for unit in gaupol.length_units:
            assert hasattr(unit, "label")

    def test_items(self):
        assert hasattr(gaupol.length_units, "CHAR")
        assert hasattr(gaupol.length_units, "EM")



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import re

MAIN = aeidon.documents.MAIN
TRAN = aeidon.documents.TRAN


class TestSearchAgent(aeidon.TestCase):

    def _test_find_next(self, pattern, docs, wrap, matches):
        self.project.set_search_target(None, docs, wrap)
        self.project.set_search_regex(pattern)
        index = None
        doc = docs[0]
        pos = None
        for match in matches:
            if match is StopIteration:
                self.raises(StopIteration,
                            self.project.find_next,
                            index, doc, pos)

                continue
            value = self.project.find_next(index, doc, pos)
            assert value == match
            index = match[0]
            doc = match[1]
            pos = match[2][1]

    def _test_find_previous(self, pattern, docs, wrap, matches):
        self.project.set_search_target(None, docs, wrap)
        self.project.set_search_regex(pattern)
        index = None
        doc = docs[-1]
        pos = None
        for match in matches:
            if match is StopIteration:
                self.raises(StopIteration,
                            self.project.find_previous,
                            index, doc, pos)

                continue
            value = self.project.find_previous(index, doc, pos)
            assert value == match
            index = match[0]
            doc = match[1]
            pos = match[2][0]

    def setup_method(self, method):
        self.project = self.new_project()
        self.delegate = self.project.find_next.im_self
        texts = (("God has promised you that\n"
                  "you will go to Heaven?"),
                 ("So you are certain of\n"
                  "being saved?"),
                 ("Be careful,\n"
                  "it's a dangerous answer."),)

        for i, text in enumerate(texts):
            self.project.subtitles[i].main_text = text
            self.project.subtitles[i].tran_text = text
        indices = range(3, len(self.project.subtitles))
        self.project.remove_subtitles(indices, register=None)

    def test_find_next__1(self):
        matches = ((0, MAIN, ( 0,  0)),
                   (0, MAIN, (26, 26)),
                   (1, MAIN, ( 0,  0)),
                   (1, MAIN, (22, 22)),
                   (2, MAIN, ( 0,  0)),
                   (2, MAIN, (12, 12)),
                   (0, MAIN, ( 0,  0)),)

        self._test_find_next(r"^", (MAIN,), True, matches)

    def test_find_next__2(self):
        matches = ((0, MAIN, (25, 25)),
                   (0, MAIN, (48, 48)),
                   (1, MAIN, (21, 21)),
                   (1, MAIN, (34, 34)),
                   (2, MAIN, (11, 11)),
                   (2, MAIN, (36, 36)),
                   (0, TRAN, (25, 25)),
                   (0, TRAN, (48, 48)),
                   (1, TRAN, (21, 21)),
                   (1, TRAN, (34, 34)),
                   (2, TRAN, (11, 11)),
                   (2, TRAN, (36, 36)),
                   (0, MAIN, (25, 25)),)

        self._test_find_next(r"$", (MAIN, TRAN), True, matches)

    def test_find_next__3(self):
        matches = ((0, TRAN, (25, 26)),
                   (1, TRAN, (21, 22)),
                   (2, TRAN, (11, 12)),
                   (0, TRAN, (25, 26)),)

        self._test_find_next(r"\n", (TRAN,), True, matches)

    def test_find_next__4(self):
        matches = ((0, MAIN, (20, 22)),
                   (0, MAIN, (37, 39)),
                   StopIteration,)

        self._test_find_next(r" t", (MAIN,), False, matches)

    def test_find_next__5(self):
        matches = ((0, TRAN, (32, 34)),
                   StopIteration,)

        self._test_find_next(r"l{2}", (TRAN,), False, matches)

    def test_find_next__6(self):
        matches = (StopIteration,)
        self._test_find_next(r"xxx", (MAIN, TRAN), False, matches)

    def test_find_next__7(self):
        matches = (StopIteration,)
        self._test_find_next(r"xxx", (MAIN, TRAN), True, matches)

    def test_find_previous__1(self):
        matches = ((2, MAIN, (12, 12)),
                   (2, MAIN, ( 0,  0)),
                   (1, MAIN, (22, 22)),
                   (1, MAIN, ( 0,  0)),
                   (0, MAIN, (26, 26)),
                   (0, MAIN, ( 0,  0)),
                   (2, MAIN, (12, 12)),)

        self._test_find_previous(r"^", (MAIN,), True, matches)

    def test_find_previous__2(self):
        matches = ((2, TRAN, (36, 36)),
                   (2, TRAN, (11, 11)),
                   (1, TRAN, (34, 34)),
                   (1, TRAN, (21, 21)),
                   (0, TRAN, (48, 48)),
                   (0, TRAN, (25, 25)),
                   (2, MAIN, (36, 36)),
                   (2, MAIN, (11, 11)),
                   (1, MAIN, (34, 34)),
                   (1, MAIN, (21, 21)),
                   (0, MAIN, (48, 48)),
                   (0, MAIN, (25, 25)),
                   (2, TRAN, (36, 36)),)

        self._test_find_previous(r"$", (MAIN, TRAN), True, matches)

    def test_find_previous__3(self):
        matches = ((2, TRAN, (11, 12)),
                   (1, TRAN, (21, 22)),
                   (0, TRAN, (25, 26)),
                   (2, TRAN, (11, 12)),)

        self._test_find_previous(r"\n", (TRAN,), True, matches)

    def test_find_previous__4(self):
        matches = ((0, MAIN, (37, 39)),
                   (0, MAIN, (20, 22)),
                   StopIteration,)

        self._test_find_previous(r" t", (MAIN,), False, matches)

    def test_find_previous__5(self):
        matches = ((0, TRAN, (32, 34)),
                   StopIteration,)

        self._test_find_previous(r"l{2}", (TRAN,), False, matches)

    def test_find_previous__6(self):
        matches = (StopIteration,)
        self._test_find_previous(r"xxx", (MAIN, TRAN), False, matches)

    def test_find_previous__7(self):
        matches = (StopIteration,)
        self._test_find_previous(r"xxx", (MAIN, TRAN), True, matches)

    @aeidon.deco.reversion_test
    def test_replace(self):
        self.project.set_search_target(None, (MAIN,))
        self.project.set_search_regex(r"\b\s")
        self.project.set_search_replacement("")
        self.project.find_next(0, MAIN, 4)
        self.project.replace()
        assert self.project.subtitles[0].main_text == (
            "God haspromised you that\n"
            "you will go to Heaven?")

    @aeidon.deco.reversion_test
    def test_replace_all(self):
        self.project.set_search_target(None, (MAIN, TRAN))
        self.project.set_search_regex(r"$")
        self.project.set_search_replacement("--")
        self.project.replace_all()
        texts = (("God has promised you that--\n"
                  "you will go to Heaven?--"),
                 ("So you are certain of--\n"
                  "being saved?--"),
                 ("Be careful,--\n"
                  "it's a dangerous answer.--"),)

        for i, text in enumerate(texts):
            assert self.project.subtitles[i].main_text == text
            assert self.project.subtitles[i].tran_text == text

    def test_set_search_regex(self):
        flags = re.DOTALL | re.MULTILINE | re.UNICODE
        finder = self.delegate._finder
        self.project.set_search_regex(r"test")
        assert finder.pattern.pattern == r"test"
        assert finder.pattern.flags == flags

    def test_set_search_regex__ignore_case(self):
        flags = re.DOTALL | re.MULTILINE | re.UNICODE
        finder = self.delegate._finder
        self.project.set_search_regex(r"test", re.IGNORECASE)
        assert finder.pattern.pattern == r"test"
        assert finder.pattern.flags == flags | re.IGNORECASE

    def test_set_search_replacement(self):
        self.project.set_search_replacement("test")
        assert self.delegate._finder.replacement == "test"

    def test_set_search_string(self):
        finder = self.delegate._finder
        self.project.set_search_string("")
        assert finder.pattern == ""
        assert finder.ignore_case == False

    def test_set_search_string__ignore_case(self):
        finder = self.delegate._finder
        self.project.set_search_string("test", True)
        assert finder.pattern == "test"
        assert finder.ignore_case == True

    def test_set_search_target(self):
        self.project.set_search_target((1,), (MAIN,), False)
        assert self.delegate._indices == (1,)
        assert self.delegate._docs == (MAIN,)
        assert self.delegate._wrap == False

    def test_set_search_target__none(self):
        self.project.set_search_target()
        assert self.delegate._indices == None
        assert self.delegate._docs == (MAIN, TRAN)
        assert self.delegate._wrap == True



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Actions that provide top-level menus for :class:`gaupol.Application`."""

import aeidon
import gaupol
_ = aeidon.i18n._


class ShowEditMenuAction(gaupol.TopMenuAction):

    """Show the "Edit" menu."""

    def __init__(self):
        """Initialize a :class:`ShowEditMenuAction` object."""
        gaupol.TopMenuAction.__init__(self, "show_edit_menu")
        self.props.label = _("_Edit")
        self.action_group = "main-safe"


class ShowFileMenuAction(gaupol.TopMenuAction):

    """Show the "File" menu."""

    def __init__(self):
        """Initialize a :class:`ShowFileMenuAction` object."""
        gaupol.TopMenuAction.__init__(self, "show_file_menu")
        self.props.label = _("_File")
        self.action_group = "main-safe"


class ShowHelpMenuAction(gaupol.TopMenuAction):

    """Show the "Help" menu."""

    def __init__(self):
        """Initialize a :class:`ShowHelpMenuAction` object."""
        gaupol.TopMenuAction.__init__(self, "show_help_menu")
        self.props.label = _("_Help")
        self.action_group = "main-safe"


class ShowProjectsMenuAction(gaupol.TopMenuAction):

    """Show the "Projects" menu."""

    def __init__(self):
        """Initialize a :class:`ShowProjectsMenuAction` object."""
        gaupol.TopMenuAction.__init__(self, "show_projects_menu")
        self.props.label = _("_Projects")
        self.action_group = "main-safe"


class ShowTextMenuAction(gaupol.TopMenuAction):

    """Show the "Text" menu."""

    def __init__(self):
        """Initialize a :class:`ShowTextMenuAction` object."""
        gaupol.TopMenuAction.__init__(self, "show_text_menu")
        self.props.label = _("_Text")
        self.action_group = "main-safe"


class ShowToolsMenuAction(gaupol.TopMenuAction):

    """Show the "Tools" menu."""

    def __init__(self):
        """Initialize a :class:`ShowToolsMenuAction` object."""
        gaupol.TopMenuAction.__init__(self, "show_tools_menu")
        self.props.label = _("T_ools")
        self.action_group = "main-safe"


class ShowViewMenuAction(gaupol.TopMenuAction):

    """Show the "View" menu."""

    def __init__(self):
        """Initialize a :class:`ShowViewMenuAction` object."""
        gaupol.TopMenuAction.__init__(self, "show_view_menu")
        self.props.label = _("_View")
        self.action_group = "main-safe"


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))
